@@ -4,9 +4,7 @@
  * GChemPaint library
  * about.cc 
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,7 +24,7 @@
 
 #include "gchempaint-config.h"
 #include "about.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
@@ -55,7 +53,7 @@ void on_about (GtkWidget* widget, void* data)
 					"authors", authors,
 					"artists", artists,
 					"comments", _("GChemPaint is a 2D chemical structures editor for Gnome"),
-					"copyright", _("(C) 2001-2005 by Jean Bréfort"),
+					"copyright", _("Copyright © 2001-2007 by Jean Bréfort"),
 					"license", license,
 					"logo", pixbuf,
 					"icon-name", "gchempaint",
