@@ -4,7 +4,7 @@
  * GChemPaint library
  * application.cc 
  *
- * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -31,24 +31,142 @@
 #include "mesomer.h"
 #include "mesomery.h"
 #include "mesomery-arrow.h"
+#include "newfiledlg.h"
 #include "reaction.h"
 #include "reactant.h"
 #include "reaction-step.h"
 #include "reaction-arrow.h"
+#include "settings.h"
+#include "theme.h"
 #include "tools.h"
 #include "window.h"
 #include "zoomdlg.h"
 #include <gcu/filechooser.h>
 #include <goffice/utils/go-file.h>
-#include <bonobo/bonobo-ui-util.h>
 #include <sys/stat.h>
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-mime-info.h>
+#include <glib/gi18n-lib.h>
 #include <locale.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
+#ifdef HAVE_FSTREAM
+#	include <fstream>
+#else
+#	include <fstream.h>
+#endif
+
+// following code is needed to get file extensions, it as been essentially copied from gnome-vfs
+static map<string, list<string> > globs;
+
+static void load_globs_from_dir (char const *directory)
+{
+  char *file_name = (char*) malloc (strlen (directory) + strlen ("/mime/globs") + 1);
+  struct stat st;
+  strcpy (file_name, directory); strcat (file_name, "/mime/globs");
+  if (stat (file_name, &st) == 0)
+    {
+  FILE *glob_file;
+  char line[255];
+
+  glob_file = fopen (file_name, "r");
+
+  if (glob_file == NULL)
+    return;
+
+  /* FIXME: Not UTF-8 safe.  Doesn't work if lines are greater than 255 chars.
+   * Blah */
+  while (fgets (line, 255, glob_file) != NULL)
+    {
+      char *colon;
+      if (line[0] == '#')
+	continue;
+
+      colon = strchr (line, ':');
+      if (colon == NULL)
+	continue;
+      *(colon++) = '\000';
+      colon[strlen (colon) -1] = '\000';
+      colon = strchr (colon, '.');
+       if (colon == NULL)
+		continue;
+     colon++;
+      if (!*colon)
+      	continue;
+      globs[line].push_back (colon);
+    }
+
+  fclose (glob_file);
+    }
+    free (file_name);
+}
+
+static void load_globs ()
+{
+  const char *xdg_data_home;
+  const char *xdg_data_dirs;
+  const char *ptr;
+
+  xdg_data_home = getenv ("XDG_DATA_HOME");
+  if (xdg_data_home)
+    {
+      load_globs_from_dir (xdg_data_home);
+    }
+  else
+    {
+      const char *home;
+
+      home = getenv ("HOME");
+      if (home != NULL)
+	{
+	  char *guessed_xdg_home;
+
+	  guessed_xdg_home = (char*) malloc (strlen (home) + strlen ("/.local/share/") + 1);
+	  strcpy (guessed_xdg_home, home);
+	  strcat (guessed_xdg_home, "/.local/share/");
+	  load_globs_from_dir (guessed_xdg_home);
+	  free (guessed_xdg_home);
+	}
+    }
+
+  xdg_data_dirs = getenv ("XDG_DATA_DIRS");
+  if (xdg_data_dirs == NULL)
+    xdg_data_dirs = "/usr/local/share/:/usr/share/";
+
+  ptr = xdg_data_dirs;
+
+  while (*ptr != '\000')
+    {
+      const char *end_ptr;
+      char *dir;
+      int len;
+ 
+      end_ptr = ptr;
+      while (*end_ptr != ':' && *end_ptr != '\000')
+	end_ptr ++;
+
+      if (end_ptr == ptr)
+	{
+	  ptr++;
+	  continue;
+	}
+
+      if (*end_ptr == ':')
+	len = end_ptr - ptr;
+      else
+	len = end_ptr - ptr + 1;
+      dir = (char*) malloc (len + 1);
+      strncpy (dir, ptr, len);
+      dir[len] = '\0';
+      load_globs_from_dir (dir);
+      free (dir);
+
+      ptr = end_ptr;
+    }
+}
 
 // Objects creation static methods
 static Object* CreateMolecule ()
@@ -106,27 +224,19 @@ bool	gcpApplication::m_bInit = false;
 bool	gcpApplication::m_Have_Ghemical = false;
 bool	gcpApplication::m_Have_InChI = false;
 
-#warning "the following lines should be removed for stable releases"
-#undef PACKAGE
-#define PACKAGE "gchempaint-unstable" 
+static void on_config_changed (GConfClient *client, guint cnxn_id, GConfEntry *entry, gcpApplication *app)
+{
+	app->OnConfigChanged (client, cnxn_id, entry);
+}
 
-static char *mime_types[] = {
-	"application/x-gchempaint",
-	"chemical/x-cml",
-	"chemical/x-mdl-molfile",
-	"chemical/x-pdb",
-	"chemical/x-chemdraw",
-};
-
-gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
+gcpApplication::gcpApplication ():
+	Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
 {
 	Apps.push_back(this);
 	m_CurZ = 6;
 	m_pActiveDoc = NULL;
 	m_pActiveTool = NULL;
-	m_pNode = NULL;
 	m_NumWindow = 1;
-//	FontName = g_strdup ("Bitstream Vera Serif 12");
 
 	if (!m_bInit) {
 		// Check for programs
@@ -184,6 +294,7 @@ gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE,
 		// Create global signal ids
 		OnChangedSignal = Object::CreateNewSignalId ();
 		OnDeleteSignal = Object::CreateNewSignalId ();
+		OnThemeChangedSignal = Object::CreateNewSignalId ();
 
 		gcpPlugin::LoadPlugins();
 		m_bInit = true;
@@ -197,11 +308,48 @@ gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE,
 	g_object_unref (G_OBJECT (IconFactory));
 	XmlDoc = xmlNewDoc ((xmlChar const*) "1.0");
 	visible_windows = 0;
+	load_globs ();
 	m_SupportedMimeTypes.push_back ("application/x-gchempaint");
-	m_SupportedMimeTypes.push_back ("chemical/x-chemdraw");
-	m_SupportedMimeTypes.push_back ("chemical/x-cml");
-	m_SupportedMimeTypes.push_back ("chemical/x-mdl-molfile");
-	m_SupportedMimeTypes.push_back ("chemical/x-pdb");
+	m_WriteableMimeTypes.push_back ("application/x-gchempaint");
+	// test if OpenBabel supports some extra types
+	TestSupportedType ("chemical/x-cml");
+	TestSupportedType ("chemical/x-mdl-molfile");
+	TestSupportedType ("chemical/x-pdb");
+	TestSupportedType ("chemical/x-xyz");
+	TestSupportedType ("chemical/x-cdx");
+	TestSupportedType ("chemical/x-cdxml");
+	TestSupportedType ("chemical/x-ncbi-asn1");
+	TestSupportedType ("chemical/x-ncbi-asn1-binary");
+	TestSupportedType ("chemical/x-ncbi-asn1-xml");
+	// now read extra types declared by the user
+	char *home = getenv ("HOME");
+	if (home) {
+		string path = home;
+		path += "/.gchempaint/mime-types";
+		char line[255];
+		ifstream f (path.c_str ());
+		while (!f.fail ()) {
+			f.getline (line, 255);
+			if (*line)
+				TestSupportedType (line);
+		}
+	}
+	
+	m_ConfClient = gconf_client_get_default ();
+	gconf_client_add_dir (m_ConfClient, "/apps/gchempaint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+	GError *error = NULL;
+	GCU_GCONF_GET (ROOTDIR"compression", int, CompressionLevel, 0)
+	GCU_GCONF_GET (ROOTDIR"tearable-mendeleiev", bool, TearableMendeleiev, false)
+	m_NotificationId = gconf_client_notify_add (m_ConfClient, "/apps/gchempaint/settings", (GConfClientNotifyFunc) on_config_changed, this, NULL, NULL);
+	// make themes permanent with this as a dummy client
+	list <string> Names = ThemeManager.GetThemesNames ();
+	list <string>::iterator j, jend = Names.end ();
+	gcpTheme *pTheme;
+	m_Dummy = new Object (0);
+	for (j = Names.begin (); j != jend; j++) {
+		pTheme = ThemeManager.GetTheme ("Default");
+		pTheme->AddClient (m_Dummy);
+	}
 }
 
 gcpApplication::~gcpApplication()
@@ -211,11 +359,10 @@ gcpApplication::~gcpApplication()
 		delete (*tool).second;
 	Tools.clear ();
 	Apps.remove (this);
-	if (m_pNode)
-		bonobo_ui_node_free (m_pNode);
 	if (XmlDoc)
 		xmlFreeDoc (XmlDoc);
 	m_SupportedMimeTypes.clear ();
+	delete m_Dummy;
 }
 
 void gcpApplication::ActivateTool(const string& toolname, bool activate)
@@ -259,49 +406,7 @@ void gcpApplication::SetStatusText(const char* text)
 
 void gcpApplication::OnSaveAs()
 {
-	FileChooser (this, true, m_SupportedMimeTypes, m_pActiveDoc);
-}
-
-void gcpApplication::AddUI (const char* filename)
-{
-	BonoboUINode *Node, *Parent = bonobo_ui_node_from_file (filename);
-	bonobo_ui_util_translate_ui  (Parent);
-	if (!m_pNode)
-		m_pNode = Parent;
-	else 
-	{
-		BonoboUINode *Child, *Next;
-		if (!strcmp (bonobo_ui_node_get_name (Parent), "Root"))
-			Node = bonobo_ui_node_children (Parent);
-		else
-			Node = NULL;
-		while (Node)
-		{
-			const char* name = bonobo_ui_node_get_name (Node);
-			char* id = bonobo_ui_node_get_attr (Node, "name");
-			Child = bonobo_ui_node_children (m_pNode);
-			while (Child)
-			{
-				char *child_id = bonobo_ui_node_get_attr (Child, "name");
-				if (!strcmp(name, bonobo_ui_node_get_name (Child))
-					&& !strcmp(id, child_id))
-				{
-					MergeNodes (Child, Node);
-					break;
-				}
-				bonobo_ui_node_free_string (child_id);
-				Child = bonobo_ui_node_next (Child);
-			}
-			bonobo_ui_node_free_string (id);
-			Next = bonobo_ui_node_next (Node);
-			if (!Child) {
-				bonobo_ui_node_unlink (Node);
-				bonobo_ui_node_add_child (m_pNode, Node);
-			}
-			Node = Next;
-		}
-		bonobo_ui_node_free (Parent);
-	}
+	FileChooser (this, true, m_WriteableMimeTypes, m_pActiveDoc);
 }
 
 enum {
@@ -324,93 +429,51 @@ bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		gtk_widget_destroy (message);
 		return true;
 	}
-	if (!mime_type)
-		mime_type = "";
-	int filetype = (mime_type)? 0: gcp_ft_Normal;
-	int image_type = CHEMISTRY;
+	int file_type = -1;
 	int n = strlen (filename), i = n - 1;
 	char const *pixbuf_type = NULL;
 	string filename2 = filename;
 	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
 	if (filename[i] == '/') i = 0;
-	ext = (i > 0)? filename + i: NULL;
-	if (!filetype) {
-		// Check mime type
-		if (!strcmp (mime_type, "application/x-gchempaint"))
-			filetype = gcp_ft_Normal;
-		else if (!strcmp (mime_type, "chemical/x-cml"))
-			filetype = gcp_ft_CML;
-		else if (!strcmp (mime_type, "chemical/x-mdl-molfile"))
-			filetype = gcp_ft_SDF;
-		else if (!strcmp (mime_type, "chemical/x-pdb"))
-			filetype = gcp_ft_PDB;
-		else if (!strcmp (mime_type, "chemical/x-chemdraw"))
-			filetype = gcp_ft_ChemDraw;
-		else if (!strcmp (mime_type, "image/svg+xml"))
-			image_type = SVG;
-		else if (!strcmp (mime_type, "image/x-eps"))
-			image_type = EPS;
-		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
-			image_type = PIXBUF;
-		if (!filetype && (!bSave || !image_type)) {
-			GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-														_("Sorry, format not supported!"));
-			gtk_dialog_run (GTK_DIALOG (message));
-			gtk_widget_destroy (message);
-			return true;
+	ext = (i > 0)? filename + i + 1: NULL;
+	list<string>::iterator it, itend = m_SupportedMimeTypes.end ();
+	for (it =  m_SupportedMimeTypes.begin (); it != itend; it++)
+		if (*it == mime_type) {
+			file_type = CHEMISTRY;
+			break;
 		}
+	if (file_type != CHEMISTRY) {
+		if (!strcmp (mime_type, "image/svg+xml"))
+			file_type = SVG;
+		else if (!strcmp (mime_type, "image/x-eps"))
+			file_type = EPS;
+		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
+			file_type = PIXBUF;
 	}
-	if (!ext)
-		if (image_type)
-			switch (image_type) {
-			case SVG:
-				ext = ".svg";
-				break;
-			case EPS:
-				ext = ".eps";
-				break; 
-			default:
-				ext = NULL;
-				break;
-			}
-		else
-			switch (filetype) {
-			case gcp_ft_Normal:
-				ext = ".gchempaint";
-				break;
-			case gcp_ft_CML:
-				ext = ".cml";
-				break;
-			case gcp_ft_SDF:
-				if (!strcmp (filename + n - 4, ".sdf")) 
-					ext = ".sdf";
-				else if (!strcmp (filename + n - 3, ".sd"))
-					ext = ".sd";
-				else if (bSave)
-					ext = ".mol";
-				break;
-			case gcp_ft_PDB:
-				if (!strcmp(filename + n - 4, ".ent"))
-					ext = ".ent";
-				else if (bSave)
-					ext = ".pdb";
-				break;
-			case gcp_ft_ChemDraw:
-				ext = ".chm";
-				break;
-			default:
-				ext = NULL;
-				break;
-			}
+	if (file_type < 0 || (!bSave && (file_type > CHEMISTRY))) {
+		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+													_("Sorry, format not supported!"));
+		gtk_dialog_run (GTK_DIALOG (message));
+		gtk_widget_destroy (message);
+		return true;
+	}
+	list<string> &exts = globs[mime_type];
 	bool err;
 	GnomeVFSURI *uri;
 	if (bSave) {
-		if (ext)
-		{
-			int i = strlen (filename) - strlen(ext);
-			if ((i <= 0) || (strcmp (filename + i, ext)))
-				filename2 += ext;
+		char const *default_ext = (exts.size ())? exts.front ().c_str (): NULL;
+		if (ext) {
+			list<string>::iterator cur, end = exts.end ();
+			for (cur = exts.begin (); cur != end; cur++)
+				if (*cur != ext) {
+					default_ext = ext;
+					break;
+				}
+			if (default_ext && strcmp (ext, default_ext))
+				ext = NULL;
 		}
+		if (default_ext && !ext)
+				filename2 += string(".") + default_ext;
 		uri = gnome_vfs_uri_new (filename2.c_str ());
 		err = gnome_vfs_uri_exists (uri);
 		gnome_vfs_uri_unref (uri);
@@ -426,8 +489,7 @@ bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		if (result == GTK_RESPONSE_YES) {
 			// destroy the old file
 			gnome_vfs_unlink (filename2.c_str ());
-			pDoc->SetType (filetype);
-			switch (image_type) {
+			switch (file_type) {
 			case SVG:
 				m_pActiveDoc->ExportImage (filename2, "svg");
 				break;
@@ -438,10 +500,10 @@ bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 				m_pActiveDoc->ExportImage (filename2, pixbuf_type, GetImageResolution ());
 				break;
 			default:
-				if (filetype == gcp_ft_Normal)
+				if (!strcmp (mime_type, "application/x-gchempaint"))
 					SaveGcp (filename2, pDoc);
 				else
-					SaveWithBabel (filename2, ext, pDoc, filetype);
+					SaveWithBabel (filename2, mime_type, pDoc);
 			}
 		}
 	} else  { //loading
@@ -449,71 +511,37 @@ bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		err = !gnome_vfs_uri_exists (uri);
 		gnome_vfs_uri_unref (uri);
 		if (err) {
-			if (!ext)
-				switch (filetype) {
-				case gcp_ft_SDF:
-					filename2 = string (filename) + ".mol";
+			if (!ext) {
+				list<string>::iterator cur, end = exts.end ();
+				for (cur = exts.begin (); cur != end; cur++) {
+					filename2 = string (filename) + "." + *cur;
 					uri = gnome_vfs_uri_new (filename2.c_str ());
 					err = !gnome_vfs_uri_exists (uri);
 					gnome_vfs_uri_unref (uri);
 					if (!err)
 						break;
-					filename2 = string (filename) + ".sdf";
-					uri = gnome_vfs_uri_new (filename2.c_str ());
-					err = !gnome_vfs_uri_exists (uri);
-					gnome_vfs_uri_unref (uri);
-					if (!err)
-						break;
-					filename2 = string (filename) + ".sd";
-					uri = gnome_vfs_uri_new (filename2.c_str ());
-					err = !gnome_vfs_uri_exists (uri);
-					gnome_vfs_uri_unref (uri);
-					if (!err)
-						break;
-					filename2 = filename;
-					break;
-				case gcp_ft_PDB:
-					filename2 = string (filename) + ".pdb";
-					uri = gnome_vfs_uri_new (filename2.c_str ());
-					err = !gnome_vfs_uri_exists (uri);
-					gnome_vfs_uri_unref (uri);
-					if (!err)
-						break;
-					filename2 = string (filename) + ".ent";
-					uri = gnome_vfs_uri_new (filename2.c_str ());
-					err = !gnome_vfs_uri_exists (uri);
-					gnome_vfs_uri_unref (uri);
-					if (!err)
-						break;
-					filename2 = filename;
-					break;
 				}
-			else
-				filename2 += ext;
-			uri = gnome_vfs_uri_new (filename2.c_str ());
-			err = !gnome_vfs_uri_exists (uri);
-			gnome_vfs_uri_unref (uri);
+			}
 			if (err) {
 				filename2 = filename;
 			}
 		}
-		if (filetype == gcp_ft_Normal)
+		if (!strcmp (mime_type, "application/x-gchempaint"))
 			OpenGcp (filename2, pDoc);
 		else
-			OpenWithBabel (filename2, ext, pDoc, filetype);
+			OpenWithBabel (filename2, mime_type, pDoc);
 	}
 	return false;
 }
 
-void gcpApplication::SaveWithBabel (string const &filename, const gchar* ext, gcpDocument* pDoc, int Type)
+void gcpApplication::SaveWithBabel (string const &filename, const gchar *mime_type, gcpDocument* pDoc)
 {
-	pDoc->SetFileName (filename, ext);
-	pDoc->SetType (Type);
+	pDoc->SetFileName (filename, mime_type);
 	pDoc->Save ();
 	GtkRecentData data;
 	data.display_name = (char*) pDoc->GetTitle ();
 	data.description = NULL;
-	data.mime_type = mime_types[Type - 1];
+	data.mime_type = (char*) mime_type;
 	data.app_name = "gchempaint";
 	data.app_exec = "gchempaint %u";
 	data.groups = NULL;
@@ -521,18 +549,20 @@ void gcpApplication::SaveWithBabel (string const &filename, const gchar* ext, gc
 	gtk_recent_manager_add_full (GetRecentManager (), filename.c_str (), &data);
 }
 
-void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gcpDocument* pDoc, int Type)
+void gcpApplication::OpenWithBabel (string const &filename, const gchar *mime_type, gcpDocument* pDoc)
 {
 	char *old_num_locale;
-	bool bNew = (pDoc == NULL), local;
+	bool bNew = (pDoc == NULL || (!pDoc->GetEmpty () && ! pDoc->GetDirty ())), local;
 	GnomeVFSFileInfo *info = NULL;
-	bool result;
+	bool result = false, read_only = false;
 	try {
 		if (!filename.length ())
 			throw (int) 0;
 		info = gnome_vfs_file_info_new ();
 		gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
 		local = GNOME_VFS_FILE_INFO_LOCAL (info);
+		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
+			read_only = true;
 		gnome_vfs_file_info_unref (info);
 		if (bNew) {
 			OnFileNew ();
@@ -549,10 +579,16 @@ void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gc
 			setlocale(LC_NUMERIC, "C");
 			OBMol Mol;
 			OBConversion Conv;
-			OBFormat* pInFormat = Conv.FormatFromExt (filename.c_str ());
-			Conv.SetInAndOutFormats (pInFormat, pInFormat);
-			Conv.Read (&Mol, &ifs);
-			result = pDoc->ImportOB(Mol);
+			OBFormat* pInFormat = Conv.FormatFromMIME (mime_type);
+			if (pInFormat == NULL)
+				throw 1;
+			Conv.SetInFormat (pInFormat);
+			while (!ifs.eof () && Conv.Read (&Mol, &ifs)) {
+				result = pDoc->ImportOB(Mol);
+				Mol.Clear ();
+				if (!result)
+					break;
+			}
 			setlocale (LC_NUMERIC, old_num_locale);
 			g_free (old_num_locale);
 			ifs.close ();
@@ -567,10 +603,15 @@ void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gc
 			OBMol Mol;
 			OBConversion Conv;
 			OBFormat* pInFormat = Conv.FormatFromExt (filename.c_str ());
-			Conv.SetInAndOutFormats (pInFormat, pInFormat);
-			Conv.Read (&Mol, &iss);
-			result = pDoc->ImportOB(Mol);
-			setlocale (LC_NUMERIC, old_num_locale);
+			if (pInFormat == NULL)
+				throw 1;
+			Conv.SetInFormat (pInFormat);
+			while (!iss.eof () && Conv.Read (&Mol, &iss)) {
+				result = pDoc->ImportOB(Mol);
+				Mol.Clear ();
+				if (!result)
+					break;
+			}
 			setlocale (LC_NUMERIC, old_num_locale);
 			g_free (old_num_locale);
 			g_free (buf);
@@ -580,14 +621,28 @@ void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gc
 			if (bNew) delete pDoc;
 			throw (int) 2;
 		}
-		pDoc->SetFileName (filename, ext);
-		pDoc->SetType (Type);
-		if (bNew)
-			AddDocument (pDoc);
+		pDoc->SetFileName (filename, mime_type);
+		pDoc->SetReadOnly (read_only);
+		double l = pDoc->GetMedianBondLength ();
+		if (l > 0.) {
+			double r = pDoc->GetBondLength () / l;
+			if (fabs (r - 1.) > .0001) { 
+				Matrix2D m (r, 0., 0., r);
+				// FIXME: this would not work for reactions
+				pDoc->Transform2D (m, 0., 0.);
+			}
+		}
+		gcpView *pView = pDoc->GetView ();
+		pView->Update (pDoc);
+		pDoc->Update ();
+		pView->EnsureSize ();
+		gcpWindow *win = pDoc->GetWindow ();
+		if (win)
+			win->SetTitle (pDoc->GetTitle ());
 		GtkRecentData data;
 		data.display_name = (char*) pDoc->GetTitle ();
 		data.description = NULL;
-		data.mime_type = mime_types[Type - 1];
+		data.mime_type = (char*) mime_type;
 		data.app_name = "gchempaint";
 		data.app_exec = "gchempaint %u";
 		data.groups = NULL;
@@ -622,7 +677,7 @@ void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gc
 
 void gcpApplication::SaveGcp (string const &filename, gcpDocument* pDoc)
 {
-	pDoc->SetFileName (filename, ".gchempaint");
+	pDoc->SetFileName (filename, "application/x-gchempaint");
 	pDoc->Save ();
 	GtkRecentData data;
 	data.display_name = (char*) pDoc->GetTitle ();
@@ -675,12 +730,12 @@ void gcpApplication::OpenGcp (string const &filename, gcpDocument* pDoc)
 		setlocale(LC_NUMERIC, "C");
 		old_time_locale = g_strdup(setlocale(LC_TIME, NULL));
 		setlocale(LC_TIME, "C");
-		if (!pDoc) {
+		if (!pDoc || !pDoc->GetEmpty () || pDoc->GetDirty ()) {
 			create = true;
 			OnFileNew ();
 			pDoc = m_pActiveDoc;
 		}
-		pDoc->SetFileName(filename, ".gchempaint");
+		pDoc->SetFileName(filename, "application/x-gchempaint");
 		bool result = pDoc->Load(xml->children);
 		setlocale(LC_NUMERIC, old_num_locale);
 		g_free(old_num_locale);
@@ -739,18 +794,6 @@ void gcpApplication::OpenGcp (string const &filename, gcpDocument* pDoc)
 	}
 }
 
-void gcpApplication::MergeNodes(BonoboUINode* Dest, BonoboUINode* Source)
-{
-	BonoboUINode *next, *child = bonobo_ui_node_children (Source);
-	while (child)
-	{
-		next = bonobo_ui_node_next (child);
-		bonobo_ui_node_unlink (child);
-		bonobo_ui_node_add_child (Dest, child);
-		child = next;
-	}
-}
-
 void gcpApplication::InitTools()
 {
 	map<string, gcpTool*>::iterator i = Tools.begin (), end = Tools.end ();
@@ -763,7 +806,7 @@ void gcpApplication::OnSaveAsImage ()
 {
 	if (!m_pActiveDoc)
 		return;
-	list<char const*> l;
+	list<string> l;
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
 		l.push_front ((*i).first.c_str ());
@@ -783,7 +826,7 @@ void gcpApplication::Zoom (double zoom)
 		if (pDialog)
 			gtk_window_present (pDialog->GetWindow ()); 
 		else
-			new gcpZoomDlg (this);
+			new gcpZoomDlg (m_pActiveDoc);
 	}
 }
 
@@ -797,7 +840,7 @@ void gcpApplication::OnToolChanged (GtkAction *current)
 	if (m_pActiveTool)
 		m_pActiveTool->Activate(false);
 	m_pActiveTool = Tools[gtk_action_get_name (current)];
-	gcpTools *Tools = dynamic_cast<gcpTools*> (Dialogs["Tools"]);
+	gcpTools *Tools = dynamic_cast<gcpTools*> (GetDialog ("tools"));
 	if (Tools)
 		Tools->OnSelectTool (m_pActiveTool);		
 	if (m_pActiveTool)
@@ -838,7 +881,7 @@ void gcpApplication::BuildTools ()
 
 void gcpApplication::ShowTools (bool visible)
 {
-	gcpTools *Tools = dynamic_cast<gcpTools*> (Dialogs["Tools"]);
+	gcpTools *Tools = dynamic_cast<gcpTools*> (GetDialog ("tools"));
 	if (!Tools) {
 		if (visible)
 			BuildTools ();
@@ -923,7 +966,8 @@ void gcpApplication::NotifyFocus (bool has_focus, gcpWindow *window)
 void gcpApplication::CloseAll ()
 {
 	while (!m_Windows.empty ())
-		(*m_Windows.begin ())->Destroy ();
+		if (!(*m_Windows.begin ())->Close ())
+			return;
 }
 
 void gcpApplication::ActivateWindowsActionWidget (const char *path, bool activate)
@@ -932,3 +976,44 @@ void gcpApplication::ActivateWindowsActionWidget (const char *path, bool activat
 	for (i = m_Windows.begin (); i != iend; i++)
 		(*i)->ActivateActionWidget (path, activate);
 }
+
+void gcpApplication::OnConfigChanged (GConfClient *client, guint cnxn_id, GConfEntry *entry)
+{
+	if (client != m_ConfClient)
+		return;	// we might want an error message?
+	if (cnxn_id != m_NotificationId)
+		return;	// we might want an error message?
+	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"compression")) {
+		CompressionLevel = gconf_value_get_int (gconf_entry_get_value (entry));
+	} else 	if (!strcmp (gconf_entry_get_key (entry),ROOTDIR"tearable-mendeleiev")) {
+		TearableMendeleiev = gconf_value_get_bool (gconf_entry_get_value (entry));
+		gcpTools *Tools = dynamic_cast<gcpTools*> (GetDialog ("tools"));
+		if (Tools)
+			Tools->Update ();
+	}
+}
+
+void gcpApplication::TestSupportedType (char const *mime_type)
+{
+	OBFormat *f = OBConversion::FormatFromMIME (mime_type);
+	if (f != NULL) {
+		m_SupportedMimeTypes.push_back (mime_type);
+		if (!(f->Flags () & NOTWRITABLE))
+			m_WriteableMimeTypes.push_back (mime_type);
+	}
+}
+
+list<string> &gcpApplication::GetExtensions(string &mime_type)
+{
+	return globs[mime_type];
+}
+
+void gcpApplication::OnThemeNamesChanged ()
+{
+	gcpNewFileDlg *dlg = dynamic_cast <gcpNewFileDlg *> (GetDialog ("newfile"));
+	if (dlg)
+		dlg->OnThemeNamesChanged ();
+	set <Document*>::iterator i, end = m_Docs.end ();
+	for (i = m_Docs.begin (); i != end; i++)
+		dynamic_cast <gcpDocument *> (*i)->OnThemeNamesChanged ();
+}
