@@ -4,7 +4,7 @@
  * GChemPaint library
  * application.h 
  *
- * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,9 +26,9 @@
 #define GCHEMPAINT_APPLICATION_H
 
 #include "tool.h"
-#include "bonobo/bonobo-ui-node.h"
 #include <gcu/application.h>
 #include <gcu/dialog.h>
+#include <gconf/gconf-client.h>
 #include <set>
 #include <string>
 #include <map>
@@ -42,6 +42,7 @@ typedef struct
 } gcpIconDesc;
 
 class gcpWindow;
+class gcpNewFileDlg;
 	
 class gcpApplication: public Application
 {
@@ -49,13 +50,10 @@ public:
 	gcpApplication();
 	virtual ~gcpApplication();
 
-	virtual void ActivateMenu(const string& menuname, bool activate);
-	virtual void ToggleMenu(const string& menuname, bool active) = 0;
-	virtual void ActivateTool(const string& toolname, bool activate);
-	virtual void ActivateToolItem(const string& itemname, bool activate);
+	void ActivateTool (const string& toolname, bool activate);
+	void ActivateWindowsActionWidget (const char *path, bool activate);
 	virtual void ClearStatus();
 	virtual void SetStatusText(const char* text);
-	void AddUI(const char* filename);
 	virtual GtkWindow* GetWindow() = 0;
 	void SetMenu(const string& menuname, GtkWidget* menu) {Menus[menuname] = menu;}
 	GtkWidget* GetMenu(const string& name) {return Menus[name];}
@@ -66,22 +64,17 @@ public:
 	void SetTool (const string& toolname, gcpTool* tool) {Tools[toolname] = tool;}
 	GtkWidget* GetToolItem(const string& name) {return ToolItems[name];}
 	void SetToolItem(const string& name, GtkWidget* w) {ToolItems[name] = w;}
-	void SetDialog(const string& name, Dialog* dialog) {Dialogs[name] = dialog;}
-	Dialog* GetDialog(const string& name) {return Dialogs[name];}
 	void SetCurZ(int Z) {m_CurZ = Z;}
 	int GetCurZ() {return m_CurZ;}
 	void OnSaveAs();
 	bool FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc = NULL);
-	void SaveWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
-	void OpenWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type);
-	void SaveGcp(const gchar* filename, gcpDocument* pDoc);
-	void OpenGcp(const gchar* filename, gcpDocument* pDoc);
-	char* GetFontName() {return FontName;}
-	void SetFontName(char* fontname);
+	void SaveWithBabel(string const &filename, const gchar *mime_type, gcpDocument* pDoc);
+	void OpenWithBabel(string const &filename, const gchar *mime_type, gcpDocument* pDoc);
+	void SaveGcp(string const &filename, gcpDocument* pDoc);
+	void OpenGcp(string const &filename, gcpDocument* pDoc);
 	xmlDocPtr GetXmlDoc() {return XmlDoc;}
 	void SetCallback(const string& name, GCallback cb) {Callbacks[name] = cb;}
 	GCallback GetCallback(const string& name) {return Callbacks[name];}
-	void OnShowElements ();
 	void OnSaveAsImage ();
 	bool HaveGhemical () {return m_Have_Ghemical;}
 	bool HaveInChI () {return m_Have_InChI;}
@@ -96,37 +89,33 @@ public:
 	void NotifyFocus (bool has_focus, gcpWindow *window = NULL);
 	void CheckFocus ();
 	void CloseAll ();
-	list<char const*> &GetSupportedMimeTypes () {return m_SupportedMimeTypes;}
+	list<string> &GetSupportedMimeTypes () {return m_SupportedMimeTypes;}
+	void OnConfigChanged (GConfClient *client,  guint cnxn_id, GConfEntry *entry);
+	list<string> &GetExtensions(string &mime_type);
+	void OnThemeNamesChanged ();
 
 	// virtual menus actions:
-	virtual void OnFileNew () = 0;
-	virtual void OnFileClose () = 0;
+	virtual void OnFileNew (char const *Theme = NULL) = 0;
 
 protected:
-//	virtual void AddDocument(gcpDocument* pDoc) = 0;
 	void InitTools();
 	void BuildTools ();
 	void ShowTools (bool visible);
 
 private:
-	void MergeNodes(BonoboUINode* Dest, BonoboUINode* Source);
+	void TestSupportedType (char const *mime_type);
 
 protected:
 	int m_CurZ;
 	gcpDocument *m_pActiveDoc;
 	gcpWindow *m_pActiveWin;
-//	std::list<gcpDocument*> m_Docs;
 	map <string, GtkWidget*> Menus;
 	map <string, GtkWidget*> ToolItems;
 	map <string, GtkWidget*> Toolbars;
 	map <string, gcpTool*> Tools;
-	map <string, Dialog*> Dialogs;
 	map <string, GCallback> Callbacks;
 	gcpTool* m_pActiveTool;
 	static bool m_bInit, m_Have_Ghemical, m_Have_InChI;
-	static map<string, string> m_SupportedPixbufFormats;
-	BonoboUINode* m_pNode;
-	char *FontName;
 	xmlDocPtr XmlDoc;
 	unsigned m_NumWindow; //used for new files (Untitled%d)
 
@@ -139,9 +128,11 @@ private:
 	unsigned m_NumDoc; //used to build the name of the action associated with the menu
 	std::set<gcpWindow*> m_Windows;
 	int visible_windows;
-	bool focused;
-	list<char const*> m_SupportedMimeTypes;
-	unsigned m_idle;
+	list<string> m_SupportedMimeTypes;
+	list<string> m_WriteableMimeTypes;
+	GConfClient *m_ConfClient;
+	guint m_NotificationId;
+	Object *m_Dummy;
 };
 
 extern list<gcpApplication*> Apps;
