@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -34,19 +34,18 @@
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
-#include "libgcpcanvas/gcp-canvas-rich-text.h"
 #include "libgcpcanvas/gcp-canvas-pango.h"
 #include <math.h>
 #include <gcu/element.h>
 #include <openbabel/mol.h>
+#include <glib/gi18n-lib.h>
 
 using namespace gcu;
 
 #define ATOM_EPSILON 0.1
 
-extern GtkTextTagTable* TextTagTable;
-
-gcpAtom::gcpAtom(): Atom()
+gcpAtom::gcpAtom(): Atom(),
+	m_ShowSymbol (false)
 {
 	m_Valence = -1; //unspecified
 	m_nlp = 0;
@@ -85,7 +84,8 @@ gcpAtom::~gcpAtom()
 		g_object_unref (G_OBJECT (m_ChargeLayout));
 }
 
-gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
+gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z),
+	m_ShowSymbol (false)
 {
 	m_ChargeAuto = false;
 	m_HPos = GetBestSide();
@@ -104,11 +104,12 @@ gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 	m_DrawCircle = false;
 }
 
-gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
+gcpAtom::gcpAtom(OBAtom* atom): Atom(),
+	m_ShowSymbol (false)
 {
-	m_x = atom->GetX() * zoom;
-	m_y = - atom->GetY() * zoom;
-	m_z = atom->GetZ() * zoom;
+	m_x = atom->GetX();
+	m_y = - atom->GetY();
+	m_z = atom->GetZ();
 	m_nlp = 0;
 	SetZ(atom->GetAtomicNum());
 	gchar* Id = g_strdup_printf("a%d", atom->GetIdx());
@@ -126,6 +127,7 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	m_ChargeAutoPos = true;
 	m_Layout = m_ChargeLayout = NULL;
 	m_DrawCircle = false;
+	m_Charge = atom->GetFormalCharge ();
 }
 
 void gcpAtom::SetZ(int Z)
@@ -278,6 +280,18 @@ void gcpAtom::Update ()
 			i++;
 		}
 		DrawCircle = nb > 1;
+		if (!DrawCircle && GetBondsNumber () == 2) {
+			i = m_Bonds.begin();
+			double angle = static_cast<gcpBond*> ((*i).second)->GetAngle2D (this);
+			i++;
+			angle -= static_cast<gcpBond*> ((*i).second)->GetAngle2D (this);
+			while (angle < 0)
+				angle += 360.;
+			while (angle > 360.)
+				angle -= 360;
+			if (fabs (angle - 180.) < 1)
+				DrawCircle = true;
+		}
 		if (DrawCircle != m_DrawCircle) {
 			m_DrawCircle = DrawCircle;
 			m_Changed = true;
@@ -305,15 +319,19 @@ void gcpAtom::Add (GtkWidget* w)
 	if (m_Layout == NULL) {
 		PangoContext* pc = pView->GetPangoContext();
 		m_Layout = pango_layout_new (pc);
+	}
+	if (m_FontName != pView->GetFontName ()) {
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
 		pango_layout_set_text (m_Layout, "l", 1);
 		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
 		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
+		m_FontName = pView->GetFontName ();
+		m_CHeight = 0.;
 	}
+	PangoRectangle rect;
 	if (m_CHeight == 0.) {
 		pango_layout_set_text (m_Layout, "C", 1);
-		PangoRectangle rect;
 		pango_layout_get_extents (m_Layout, &rect, NULL);
 		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 	}
@@ -331,6 +349,9 @@ void gcpAtom::Add (GtkWidget* w)
 		int sw, sp;
 		const gchar* symbol = GetSymbol (), *text;
 		sw = strlen (symbol);
+		pango_layout_set_text (m_Layout, symbol, sw);
+		pango_layout_get_extents (m_Layout, &rect, NULL);
+		m_width += rect.width / PANGO_SCALE;
 		int n = GetAttachedHydrogens ();
 		if (n > 0) {
 			if (n > 1) {
@@ -372,7 +393,6 @@ void gcpAtom::Add (GtkWidget* w)
 			sp = 0;
 			pango_layout_set_text (m_Layout, text, -1);
 		}
-		PangoRectangle rect;
 		pango_layout_get_extents (m_Layout, NULL, &rect);
 		m_length = double (rect.width / PANGO_SCALE);
 		m_text_height = m_height = rect.height / PANGO_SCALE;
@@ -470,25 +490,25 @@ void gcpAtom::Add (GtkWidget* w)
 		switch (align) {
 		case -2:
 			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-			y += pTheme->GetChargeSignHeight () / 2.;
+			y += pTheme->GetChargeSignSize () / 2.;
 			break;
 		case -1:
-			xc = x - pTheme->GetChargeSignSize () -1.;
+			xc = x - pTheme->GetChargeSignSize () - pTheme->GetPadding ();
 			break;
 		case 0:
 		case -3:
 			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 			break;
 		case 1:
-			xc = x + m_ChargeWidth + 1.;
+			xc = x + m_ChargeWidth + pTheme->GetPadding ();
 			break;
 		case 2:
 			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-			y -= pTheme->GetChargeSignHeight () / 2.;
+			y -= pTheme->GetChargeSignSize () / 2.;
 			break;
 		}
 		x = xc - 1.;
-		yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
+		yc = y - pTheme->GetChargeSignSize () / 2.;
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -564,10 +584,27 @@ void gcpAtom::Update(GtkWidget* w)
 	x *= pTheme->GetZoomFactor ();
 	y *= pTheme->GetZoomFactor ();
 	GnomeCanvasGroup *group = pData->Items[this];
+	if (m_FontName != pData->View->GetFontName ()) {
+		gcpView *pView = pData->View;
+		PangoContext* pc = pView->GetPangoContext();
+		PangoLayout *Layout = pango_layout_new (pc);
+		pango_layout_set_font_description (Layout, pView->GetPangoFontDesc ());
+		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
+		pango_layout_set_text (Layout, "l", 1);
+		PangoLayoutIter* iter = pango_layout_get_iter (Layout);
+		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
+		m_FontName = pView->GetFontName ();
+		pango_layout_set_text (Layout, "C", 1);
+		PangoRectangle rect;
+		pango_layout_get_extents (Layout, &rect, NULL);
+		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		g_object_unref (G_OBJECT (Layout));
+	}
 	if (m_Changed) {
 		BuildItems (pData);
 	} else {
-		if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
+		if ((GetZ() != 6) || (GetBondsNumber() == 0) || m_ShowSymbol) {
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "symbol")),
 								"x", x - m_lbearing,
 								"y", y - m_ascent + m_CHeight,
@@ -638,25 +675,25 @@ void gcpAtom::Update(GtkWidget* w)
 			switch (align) {
 			case -2:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-				y += pTheme->GetChargeSignHeight () / 2.;
+				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				xc = x - pTheme->GetChargeSignSize () -1.;
+				xc = x - pTheme->GetChargeSignSize () - pTheme->GetPadding ();
 				break;
 			case 0:
 			case -3:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
-				xc = x + m_ChargeWidth + 1.;
+				xc = x + m_ChargeWidth + pTheme->GetPadding ();
 				break;
 			case 2:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-				y -= pTheme->GetChargeSignHeight () / 2.;
+				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
-			x = xc - 1.;
-			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
+			x = xc -1.;
+			yc = y - pTheme->GetChargeSignSize () / 2.;
 			if (fig) {
 				g_object_set (G_OBJECT (figure),
 							"layout", m_ChargeLayout,
@@ -717,7 +754,7 @@ void gcpAtom::Update(GtkWidget* w)
 				PangoRectangle rect;
 				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				m_ChargeTWidth = m_ChargeWidth + 2 + pTheme->GetChargeSignSize ();
+				m_ChargeTWidth = m_ChargeWidth + pTheme->GetPadding () + pTheme->GetChargeSignSize ();
 			} else {
 				m_ChargeWidth = 0.;
 				m_ChargeTWidth = pTheme->GetChargeSignSize ();
@@ -725,25 +762,25 @@ void gcpAtom::Update(GtkWidget* w)
 			switch (align) {
 			case -2:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-				y += pTheme->GetChargeSignHeight () / 2.;
+				y += pTheme->GetChargeSignSize () / 2.;
 				break;
 			case -1:
-				xc = x - pTheme->GetChargeSignSize () -1.;
+				xc = x - pTheme->GetChargeSignSize () - pTheme->GetPadding ();
 				break;
 			case 0:
 			case -3:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
-				xc = x + m_ChargeWidth + 1.;
+				xc = x + m_ChargeWidth + pTheme->GetPadding ();
 				break;
 			case 2:
 				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
-				y -= pTheme->GetChargeSignHeight () / 2.;
+				y -= pTheme->GetChargeSignSize () / 2.;
 				break;
 			}
 			x = xc - 1.;
-			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
+			yc = y - pTheme->GetChargeSignSize () / 2.;
 			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 						group,
 						gnome_canvas_group_ext_get_type(),
@@ -981,18 +1018,18 @@ int gcpAtom::GetChargePosition(unsigned char& Pos, double Angle, double& x, doub
 		y = m_y + m_height / 2.0;
 		return -2;
 	case CHARGE_E:
-		x = m_x + 12. + m_width / 2.0;
+		x = m_x /*+ 12.*/ + m_width / 2.0;
 		y = m_y;
 		return 1;
 	case CHARGE_W:
-		x = m_x - 12. - m_width / 2.0;
+		x = m_x /*- 12.*/ - m_width / 2.0;
 		y = m_y;
 		return -1;
 	default: {
 			double t = tan (Angle / 180. * M_PI);
 			double limit = atan (m_height / m_width) * 180. / M_PI;
 			if (Angle < limit) {
-				x = m_x +  12. + m_width / 2.;
+				x = m_x /*+  12. */+ m_width / 2.;
 				y = m_y - m_width / 2. * t;
 				return 1;
 			} else if (Angle < 180. - limit) {
@@ -1003,7 +1040,7 @@ int gcpAtom::GetChargePosition(unsigned char& Pos, double Angle, double& x, doub
 				y = m_y - m_height / 2.;
 				return 2;
 			} else if (Angle < 180. + limit) {
-				x = m_x - 12. - m_width / 2.;
+				x = m_x /*- 12.*/ - m_width / 2.;
 				y = m_y + m_width / 2. * t;
 				return -1;
 			} else if (Angle < 360. - limit) {
@@ -1014,7 +1051,7 @@ int gcpAtom::GetChargePosition(unsigned char& Pos, double Angle, double& x, doub
 				y = m_y + m_height / 2.;
 				return -2;
 			} else {
-				x = m_x +  12. + m_width / 2.;
+				x = m_x /*+  12.*/ + m_width / 2.;
 				y = m_y - m_width / 2. * t;
 				return 1;
 			}
@@ -1184,10 +1221,14 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 	GetCoords (&x, &y);
 	x *= pTheme->GetZoomFactor ();
 	y *= pTheme->GetZoomFactor ();
-	if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
+	if ((GetZ() != 6) || (GetBondsNumber() == 0) || m_ShowSymbol) {
 		int sw, sp;
 		const gchar* symbol = GetSymbol (), *text;
+		PangoRectangle rect;
 		sw = strlen (symbol);
+		pango_layout_set_text (m_Layout, symbol, sw);
+		pango_layout_get_extents (m_Layout, &rect, NULL);
+		m_width += rect.width / PANGO_SCALE;
 		int n = GetAttachedHydrogens ();
 		PangoAttrList *pal = pango_attr_list_new ();
 		if (n > 0) {
@@ -1229,7 +1270,6 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 			sp = 0;
 			pango_layout_set_text (m_Layout, text, -1);
 		}
-		PangoRectangle rect;
 		pango_layout_get_extents (m_Layout, NULL, &rect);
 		m_length = double (rect.width / PANGO_SCALE);
 		m_text_height = m_height = rect.height / PANGO_SCALE;
@@ -1470,6 +1510,9 @@ xmlNodePtr gcpAtom::Save (xmlDocPtr xml)
 			g_free (buf);
 		}
 	}
+	if (GetZ () == 6 && m_ShowSymbol) {
+		xmlNewProp (node, (xmlChar*) "show-symbol", (xmlChar*) "true");
+	}
 	return node;
 }
 	
@@ -1537,6 +1580,12 @@ bool gcpAtom::Load (xmlNodePtr node)
 		m_ChargeAutoPos = false;
 	} else
 		m_ChargeDist = 0.;
+	buf = (char*) xmlGetProp (node, (xmlChar*) "show-symbol");
+	if (buf) {
+		if (!strcmp (buf, "true"))
+			m_ShowSymbol = true;
+		xmlFree (buf);
+	}
 	return true;
 }
 	
@@ -1615,3 +1664,37 @@ void gcpAtom::Transform2D (Matrix2D& m, double x, double y)
 		}
 	}
 }
+
+static void do_display_symbol (GtkToggleAction *action, gcpAtom* Atom)
+{
+	gcpDocument *Doc = static_cast <gcpDocument *> (Atom->GetDocument ());
+	gcpOperation *Op = Doc->GetNewOperation (GCP_MODIFY_OPERATION);
+	Object *Obj = Atom->GetGroup ();
+	Op->AddObject (Obj, 0);
+	Atom->SetShowSymbol (gtk_toggle_action_get_active (action));
+	Atom->Update();
+	Atom->ForceChanged ();
+	Atom->EmitSignal (OnChangedSignal);
+	Op->AddObject (Obj, 1);
+	Doc->FinishOperation ();
+	Doc->GetView ()->Update (Atom);
+}
+
+bool gcpAtom::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
+{
+	bool result = false;
+	if (GetZ () == 6 && GetBondsNumber() != 0) {
+		GtkActionGroup *group = gtk_action_group_new ("atom");
+		GtkAction *action;
+		action = gtk_action_new ("Atom", _("Atom"),NULL, NULL);
+		gtk_action_group_add_action (group, action);
+		action = GTK_ACTION (gtk_toggle_action_new ("show-symbol", _("Display symbol"),  _("Whether to display carbon atom symbol or not"), NULL));
+		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), m_ShowSymbol);
+		g_signal_connect (action, "toggled", G_CALLBACK (do_display_symbol), this);
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Atom'><menuitem action='show-symbol'/></menu></popup></ui>", -1, NULL);
+		gtk_ui_manager_insert_action_group (UIManager, group, 0);
+		result = true;
+	}
+	return result | GetParent ()->BuildContextualMenu (UIManager, object);
+}
