@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.h 
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,6 +29,7 @@
 #include <glib.h>
 #include <gcu/atom.h>
 #include <gcu/element.h>
+#include <gcu/macros.h>
 #include <libgnomecanvas/gnome-canvas.h>
 #include "widgetdata.h"
 
@@ -70,7 +71,7 @@ class gcpAtom: public Atom
 public:
 	gcpAtom();
 	gcpAtom(int Z, double x, double y, double z);
-	gcpAtom(OBAtom* atom, double zoom = 1.0);
+	gcpAtom(OBAtom* atom);
 	virtual ~gcpAtom();
 
 public :
@@ -95,6 +96,7 @@ public :
 	virtual bool AcceptCharge (int charge);
 	virtual double GetYAlign ();
 	virtual void Transform2D (Matrix2D& m, double x, double y);
+	bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
 	/*!
 	@param Mol: a pointer to a molecule
 
@@ -108,6 +110,8 @@ public :
 	void SetChargePosition (unsigned char Pos, bool def, double angle = 0., double distance = 0.);
 	char GetChargePosition (double *Angle, double *Dist);
 	void SetCharge (int charge);
+	int GetCharge () {return m_Charge;}
+	void ForceChanged () {m_Changed = true;}
 
 private:
 	void BuildItems (gcpWidgetData* pData);
@@ -141,6 +145,9 @@ private:
 	angle between the two bonds and second value is the direction */
 	PangoLayout *m_Layout, *m_ChargeLayout;
 	bool m_DrawCircle;
+	string m_FontName;
+
+GCU_PROP (bool, ShowSymbol)
 };
 
 #endif // GCHEMPAINT_ATOM_H
