@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -36,6 +36,7 @@
 #include "libgcpcanvas/gcp-canvas-line.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-polygon.h"
+#include <glib/gi18n-lib.h>
 #include <math.h>
 
 gcpBond::gcpBond(): Bond()
@@ -370,6 +371,8 @@ double gcpBond::Get2DLength()
 
 void gcpBond::SetSelected(GtkWidget* w, int state)
 {
+	if (!m_order)
+		return;
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar* color;
@@ -454,10 +457,10 @@ void gcpBond::Add(GtkWidget* w)
 			gnome_canvas_item_lower_to_bottom (item);
 			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 			Atom *pAtom = GetAtom (0);
-			if (pAtom->GetZ () != 6)
+			if (pAtom->GetZ () != 6 || static_cast <gcpAtom*> (pAtom)->GetShowSymbol ())
 				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
 			pAtom = GetAtom (1);
-			if (pAtom->GetZ () != 6)
+			if (pAtom->GetZ () != 6 || static_cast <gcpAtom*> (pAtom)->GetShowSymbol ())
 				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
 			gnome_canvas_path_def_unref (path);
 		}
@@ -494,12 +497,10 @@ void gcpBond::Add(GtkWidget* w)
 	pData->Items[this] = group;
 	if (pAtom0->GetParent()->GetType() == FragmentType)
 		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(pData->Items[pAtom0->GetParent()]));
-	else if (pAtom0->GetZ() != 6)
+	else if (pAtom0->GetZ() != 6 || static_cast <gcpAtom*> (pAtom0)->GetShowSymbol ())
 		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(pData->Items[pAtom0]));
-	else {
+	else
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM(pData->Items[pAtom0]));
-		gnome_canvas_item_raise (GNOME_CANVAS_ITEM (pData->Items[pAtom0]), 1);
-	}
 	map<string, Object*>::iterator i;
 	Object* electron = pAtom0->GetFirstChild (i);
 	while (electron){ 
@@ -508,12 +509,10 @@ void gcpBond::Add(GtkWidget* w)
 	}
 	if (pAtom1->GetParent()->GetType() == FragmentType)
 		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(pData->Items[pAtom1->GetParent()]));
-	else if (pAtom1->GetZ() != 6)
+	else if (pAtom1->GetZ() != 6 || static_cast <gcpAtom*> (pAtom1)->GetShowSymbol ())
 		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(pData->Items[pAtom1]));
-	else {
+	else
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (pData->Items[pAtom1]));
-		gnome_canvas_item_raise (GNOME_CANVAS_ITEM (pData->Items[pAtom1]), 1);
-	}
 	electron = pAtom1->GetFirstChild (i);
 	while (electron){ 
 		gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(pData->Items[electron]));
@@ -711,7 +710,8 @@ GnomeCanvasPathDef *gcpBond::BuildCrossingPathDef (gcpWidgetData* pData)
 
 void gcpBond::Update(GtkWidget* w)
 {
-	if (!w) return;
+	if (!w || !m_order)
+		return;
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data(G_OBJECT(w), "data");
 	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	bool result = false;
@@ -753,10 +753,10 @@ void gcpBond::Update(GtkWidget* w)
 				gnome_canvas_item_lower_to_bottom (item);
 				gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 				Atom *pAtom = GetAtom (0);
-				if (pAtom->GetZ () != 6)
+				if (pAtom->GetZ () != 6 || static_cast <gcpAtom*> (pAtom)->GetShowSymbol ())
 					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
 				pAtom = GetAtom (1);
-				if (pAtom->GetZ () != 6)
+				if (pAtom->GetZ () != 6 || static_cast <gcpAtom*> (pAtom)->GetShowSymbol ())
 					gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (pData->Items[pAtom]));
 			}
 			gnome_canvas_path_def_unref (path);
@@ -767,6 +767,8 @@ void gcpBond::Update(GtkWidget* w)
 	path = BuildPathDef (pData);
 	obj = g_object_get_data(G_OBJECT(group), "path");
 	g_object_set (obj, "bpath", path, NULL);
+	if (m_type == NormalBondType || m_type == UndeterminedBondType)
+		g_object_set (obj, "width_units", pTheme->GetBondWidth (), NULL);
 	gnome_canvas_path_def_unref (path);
 }
 	
