@@ -4,7 +4,7 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,23 +35,28 @@
 #include "text.h"
 #include "theme.h"
 #include "window.h"
+#include <goffice/utils/go-rangefunc.h>
 #include <locale.h>
 #include <unistd.h>
 #include <libgen.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
+#include <glib/gi18n-lib.h>
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
+#include <stack>
 
 using namespace OpenBabel;
 using namespace std;
 
 SignalId OnChangedSignal;
 SignalId OnDeleteSignal;
+SignalId OnThemeChangedSignal;
 
 gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window):
 	Document (App),
+	m_FileType ("application/x-gchempaint"),
 	m_OpID (0),
 	m_LastStackSize (0)
 {
@@ -63,11 +68,10 @@ gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window
 	m_title = NULL;
 	m_bWriteable = true;
 	m_PangoAttrList = pango_attr_list_new ();
+	m_Theme = NULL;
 	SetTheme (ThemeManager.GetTheme ("Default"));
 	m_pView = new gcpView(this, !StandAlone);
-	m_bDirty = m_bIsLoading = m_bUndoRedo = false;
-	m_FileType = gcp_ft_Normal;
-	m_DocPropDlg = NULL;
+	m_bIsLoading = m_bUndoRedo = false;
 	g_date_set_time(&CreationDate, time(NULL));
 	g_date_clear(&RevisionDate, 1);
 	const char* chn = getenv ("REAL_NAME");
@@ -98,8 +102,6 @@ gcpDocument::~gcpDocument ()
 	if (m_pCurOp)
 		delete m_pCurOp;
 	m_pCurOp = NULL;
-	if (m_DocPropDlg)
-		m_DocPropDlg->Destroy();
 	if (m_filename)
 		g_free (m_filename);
 	if (m_title)
@@ -119,11 +121,11 @@ gcpDocument::~gcpDocument ()
 		obj->Lock ();
 		Remove (obj);
 	}
-	if (m_pApp)
-		m_pApp->RemoveDocument (this);
 	if (m_pView)
 		delete m_pView;
 	pango_attr_list_unref (m_PangoAttrList);
+	if (m_Theme)
+		m_Theme->RemoveClient (this);
 }
 
 GtkWidget* gcpDocument::GetWidget()
@@ -143,11 +145,12 @@ const gchar* gcpDocument::GetLabel()
 	return m_label;
 }
 
-void gcpDocument::SetFileName (string const &Name, const gchar* ext)
+void gcpDocument::SetFileName (string const &Name, const gchar* mime_type)
 {
 	if (m_filename)
 		g_free (m_filename);
 	m_filename = g_strdup (Name.c_str ());
+	m_FileType = mime_type;
 	char *path = g_path_get_dirname (m_filename);
 	m_pApp->SetCurDir (path);
 	g_free (path);
@@ -160,12 +163,16 @@ void gcpDocument::SetFileName (string const &Name, const gchar* ext)
 		j--;
 	if (m_label)
 		g_free (m_label);
-	if (strcmp (m_filename + j, ext))
+	char *ext = m_filename + j + 1;
+	list<string> &exts = m_pApp->GetExtensions (m_FileType);
+	list<string>::iterator cur, end = exts.end ();
+	for (cur = exts.begin (); cur != end; cur++)
+		if (*cur == ext) {
+			m_label = g_strndup (m_filename + i, j - i);
+			break;
+		}
+	if (!m_label)
 		m_label = g_strdup (m_filename + i);
-	else
-		m_label = g_strndup (m_filename + i, j - i);
-	if (m_pView)
-		m_pView->UpdateLabel (m_label);
 }
 
 void gcpDocument::BuildBondList(list<gcpBond*>& BondList, Object* obj)
@@ -187,84 +194,51 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	if (m_comment) {g_free(m_comment) ; m_comment = NULL;}
 	g_date_clear(&CreationDate, 1);
 	g_date_clear(&RevisionDate, 1);
-	OBBond *bond;
-	vector<OBEdgeBase*>::iterator j;
-	bond = Mol.BeginBond(j);
-	double l = (bond)? floor(m_BondLength / bond->GetLength()): 1.;
-	if (l == 0.) l = 1.;
-	m_title = g_strdup(Mol.GetTitle());
+	m_title = g_strdup(Mol.GetTitle()); // Hmm, and if there are several molecules?
 	OBAtom *atom;
 	gcpAtom* pAtom;
-	double xmin = DBL_MAX, xmax = 0., ymin = DBL_MAX, ymax = 0., x, y;
-	bool valid = false;
 	vector<OBNodeBase*>::iterator i;
 	for (atom = Mol.BeginAtom(i); atom; atom = Mol.NextAtom(i))
 	{
 		if (atom->GetAtomicNum () == 0)
 			continue;
-		AddAtom(pAtom = new gcpAtom(atom, l));
-		pAtom->GetCoords((double*)&x, (double*)&y);
-		if (valid)
-		{
-			if (xmin > x) xmin = x;
-			else if (xmax < x) xmax = x;
-			if (ymin > y) ymin = y;
-			else if (ymax < y) ymax = y;
-		}
-		else
-		{
-			valid = true;
-			xmin = xmax = x;
-			ymin = ymax = y;
-		}
-	}
-	x = (xmin < 50)? 50 - xmin: 0;//FIXME: 50 should be replaced by something more intelligent!
-	y = (ymin < 50)? 50 - ymin: 0;
-	if (valid && ((x != 0.0) || (y != 0.0)))
-	{
-		Object* pObject;
-		map<string, Object*>::iterator a;
-		for (pObject = GetFirstChild(a); pObject; pObject = GetNextChild(a))
-			pObject->Move(x, y);
+		AddAtom(pAtom = new gcpAtom(atom));
 	}
 	gcpAtom *begin, *end;
 	gcpBond *pBond;
 	unsigned char order;
 	gchar* Id;
-	for (; bond; bond = Mol.NextBond(j))
-	{
-		Id = g_strdup_printf("a%d", bond->GetBeginAtomIdx());
-		begin = (gcpAtom*)GetDescendant(Id);//Don't verify it is really an atom?
-		g_free(Id);
-		Id = g_strdup_printf("a%d", bond->GetEndAtomIdx());
-		end = (gcpAtom*)GetDescendant(Id);//Don't verify it is really an atom?
-		g_free(Id);
+	OBBond *bond;
+	vector<OBEdgeBase*>::iterator j;
+	for (bond = Mol.BeginBond (j); bond; bond = Mol.NextBond (j)) {
+		Id = g_strdup_printf ("a%d", bond->GetBeginAtomIdx ());
+		begin = (gcpAtom*) GetDescendant (Id);//Don't verify it is really an atom?
+		g_free (Id);
+		Id = g_strdup_printf ("a%d", bond->GetEndAtomIdx ());
+		end = (gcpAtom*) GetDescendant (Id);//Don't verify it is really an atom?
+		g_free (Id);
 		if (!end)
 			continue;
-		order = (unsigned char)(bond->GetBO());
-		if ((pBond = (gcpBond*)begin->GetBond(end)) != NULL)
-		{
-			pBond->IncOrder(order);
-			m_pView->Update(pBond);
-			m_pView->Update(begin);
-			m_pView->Update(end);
-		}
-		else
-		{
-			Id = g_strdup_printf("b%d", bond->GetIdx());
-			pBond = new gcpBond(begin, end, order);
-			if (bond->IsWedge()) pBond->SetType(UpBondType);
-			else if (bond->IsHash()) pBond->SetType(DownBondType);
-			pBond->SetId(Id);
-			g_free(Id);
-			AddBond(pBond);
+		order = (unsigned char) (bond->GetBO ());
+		if ((pBond = (gcpBond*) begin->GetBond (end)) != NULL) {
+			pBond->IncOrder (order);
+			m_pView->Update (pBond);
+			m_pView->Update (begin);
+			m_pView->Update (end);
+		} else {
+			Id = g_strdup_printf ("b%d", bond->GetIdx());
+			pBond = new gcpBond (begin, end, order);
+			if (bond->IsWedge ())
+				pBond->SetType (UpBondType);
+			else if (bond->IsHash ())
+				pBond->SetType (DownBondType);
+			pBond->SetId (Id);
+			g_free (Id);
+			AddBond (pBond);
 		}
 	}
 	if (m_Window)
 		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
-	m_pView->Update (this);
-	Update ();
-	m_pView->EnsureSize ();
 	return true;
 }
 
@@ -288,56 +262,96 @@ void gcpDocument::ExportOB()
 	unsigned index = 1;
 	double x, y, z;
 	gchar *old_num_locale;
-	Mol.BeginModify();
-	BuildAtomTable(AtomTable, this, index);
-	Mol.ReserveAtoms(AtomTable.size());
-	Mol.SetTitle((char*)GetTitle());
-	Mol.SetDimension (2);
-	for (i = AtomTable.begin(); i != AtomTable.end(); i++)
-	{
-		pgAtom = (gcpAtom*)GetDescendant((*i).first.data());
-		Atom.SetIdx((*i).second);
-		Atom.SetAtomicNum(pgAtom->GetZ());
-		pgAtom->GetCoords(&x, &y, &z);
-		Atom.SetVector(x / 100, 4 - y / 100, z / 100);
-		Mol.AddAtom(Atom);
-		Atom.Clear();
-	}
-	BuildBondList(BondList, this);
-	list<gcpBond*>::iterator j;
-	int start, end, order, flag;
-	for (j = BondList.begin(); j != BondList.end(); j++)
-	{
-		order = (*j)->GetOrder();
-		start = AtomTable[(*j)->GetAtom(0)->GetId()];
-		end = AtomTable[(*j)->GetAtom(1)->GetId()];
-		switch((*j)->GetType())
-		{
-			case UpBondType:
-				flag = OB_WEDGE_BOND;
-				break;
-			case DownBondType:
-				flag = OB_HASH_BOND;
-				break;
-			default:
-				flag = 0;
-		}
-		Mol.AddBond(start, end, order, flag);
-	}
-	Mol.EndModify();
+	map< string, Object * >::iterator m;
+	stack<map< string, Object * >::iterator> iters;
+	set<Object *> Mols;
+	Object *Cur = this, *Ob;
 	try {
 		ostringstream ofs;
 		GnomeVFSHandle *handle = NULL;
 		GnomeVFSFileSize n;
 		GnomeVFSResult res;
-		if ((res = gnome_vfs_create(&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0644)) != GNOME_VFS_OK)
+		res = gnome_vfs_open (&handle, m_filename, GNOME_VFS_OPEN_WRITE);
+		if (res == GNOME_VFS_ERROR_NOT_FOUND)
+			res = gnome_vfs_create (&handle, m_filename, GNOME_VFS_OPEN_WRITE, true, 0666);
+		if (res != GNOME_VFS_OK)
 			throw (int) res;
 		old_num_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
 		setlocale(LC_NUMERIC, "C");
 		OBConversion Conv;
-		OBFormat* pOutFormat = Conv.FormatFromExt (m_filename);
-		Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
-		Conv.Write (&Mol, &ofs);
+		OBFormat* pOutFormat = Conv.FormatFromMIME (m_FileType.c_str ());
+		if (pOutFormat != NULL) {
+			Conv.SetOutFormat (pOutFormat);
+			Ob = GetFirstChild (m);
+			while (Ob) {
+				if (Ob->GetType () == MoleculeType)
+					Mols.insert (Ob);
+				else if (Ob->HasChildren ()) {
+					Cur = Ob;
+					iters.push (m);
+					Ob = Cur->GetFirstChild (m);
+					continue;
+				}
+				Ob = Cur->GetNextChild (m);
+				while (!Ob && !iters.empty ()) {
+					m = iters.top ();
+					iters.pop ();
+					Cur = Cur->GetParent ();
+					Ob = Cur->GetNextChild (m);
+				}
+			}
+			set<Object *>::iterator mi, mend = Mols.end ();
+			unsigned nb = 1;
+			Conv.SetOneObjectOnly (false);
+			for (mi = Mols.begin (); mi != mend; mi++)
+			{
+				Ob = *mi;
+				if (nb == Mols.size ())
+					Conv.SetOneObjectOnly (true);
+				Mol.BeginModify ();
+				index = 1;
+				BuildAtomTable (AtomTable, Ob, index);
+				Mol.ReserveAtoms (AtomTable.size ());
+				Mol.SetTitle ((char*) GetTitle ());
+				Mol.SetDimension (2);
+				for (i = AtomTable.begin (); i != AtomTable.end (); i++) {
+					pgAtom = (gcpAtom*) Ob->GetDescendant ((*i).first.data ());
+					Atom.SetIdx ((*i).second);
+					Atom.SetAtomicNum (pgAtom->GetZ ());
+					pgAtom->GetCoords (&x, &y, &z);
+					Atom.SetVector (x / 100., 4. - y / 100., z / 100.);
+					Atom.SetFormalCharge (pgAtom->GetCharge ());
+					Mol.AddAtom (Atom);
+					Atom.Clear ();
+				}
+				BuildBondList (BondList, Ob);
+				list<gcpBond*>::iterator j;
+				int start, end, order, flag;
+				for (j = BondList.begin (); j != BondList.end (); j++)
+				{
+					order = (*j)->GetOrder ();
+					start = AtomTable[(*j)->GetAtom (0)->GetId ()];
+					end = AtomTable[(*j)->GetAtom (1)->GetId ()];
+					switch ((*j)->GetType ()) {
+					case UpBondType:
+						flag = OB_WEDGE_BOND;
+						break;
+					case DownBondType:
+						flag = OB_HASH_BOND;
+						break;
+					default:
+						flag = 0;
+					}
+					Mol.AddBond (start, end, order, flag);
+				}
+				Mol.EndModify ();
+				Conv.SetOutputIndex (nb++);
+				Conv.Write (&Mol, &ofs);
+				Mol.Clear ();
+				AtomTable.clear ();
+				BondList.clear ();
+			}
+		}
 		setlocale(LC_NUMERIC, old_num_locale);
 		g_free(old_num_locale);
 		if ((res = gnome_vfs_write (handle, ofs.str ().c_str (), (GnomeVFSFileSize) ofs.str ().size (), &n)) != GNOME_VFS_OK)
@@ -382,7 +396,6 @@ void gcpDocument::AddAtom(gcpAtom* pAtom)
 	mol->SetId(id);
 	AddChild(mol);
 	mol->AddAtom(pAtom);
-	m_bDirty = true;
 }
 
 void gcpDocument::AddFragment(gcpFragment* pFragment)
@@ -417,7 +430,6 @@ void gcpDocument::AddFragment(gcpFragment* pFragment)
 		mol->AddFragment(pFragment);
 	}
 	pFragment->AnalContent();
-	m_bDirty = true;
 }
 
 void gcpDocument::AddBond(gcpBond* pBond)
@@ -473,7 +485,6 @@ void gcpDocument::AddBond(gcpBond* pBond)
 		pMol0->SetId(id);
 		AddChild(pMol0);
 	}
-	m_bDirty = true;
 }
 
 static int cb_xml_to_vfs (GnomeVFSHandle *handle, const char* buf, int nb)
@@ -495,16 +506,18 @@ void gcpDocument::Save()
 	setlocale (LC_TIME, "C");
 
 	try {
-		if (m_FileType != gcp_ft_Normal)
+		if (m_FileType != "application/x-gchempaint")
 			ExportOB ();
 		else {
 			xml = BuildXMLTree();
+			xmlSetDocCompressMode (xml, CompressionLevel);
+			
 			xmlIndentTreeOutput = true;
 			xmlKeepBlanksDefault(0);
 		
 			GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
 			gnome_vfs_get_file_info (m_filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
-		
+
 			if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
 				gnome_vfs_file_info_unref (info);
 				if (xmlSaveFormatFile (m_filename, xml, true) < 0) /*Error(SAVE)*/;
@@ -526,7 +539,9 @@ void gcpDocument::Save()
 				SetReadOnly (false);
 			}
 		}
-		m_bDirty = false;
+		SetDirty (false);
+		m_LastStackSize = m_UndoList.size ();
+		m_OpID = m_UndoList.front ()->GetID ();
 	}
 	catch (int num) {
 		if (xml)
@@ -560,7 +575,6 @@ bool gcpDocument::Load (xmlNodePtr root)
 	}
 	g_date_clear (&CreationDate, 1);
 	g_date_clear (&RevisionDate, 1);
-	m_FileType = gcp_ft_Normal;
 	xmlNodePtr node, child;
 	char* tmp;
 	Object* pObject;
@@ -617,6 +631,22 @@ bool gcpDocument::Load (xmlNodePtr root)
 		}
 	}
 
+	node = GetNodeByName (root, "theme");
+	if (node) {
+		gcpTheme *pTheme = new gcpTheme (NULL), *pLocalTheme;
+		pTheme->Load (node);
+		pLocalTheme = ThemeManager.GetTheme (_(pTheme->GetName ().c_str ()));
+		if (!pLocalTheme)
+			pLocalTheme = ThemeManager.GetTheme (pTheme->GetName ().c_str ());
+		if (pLocalTheme && *pLocalTheme == *pTheme) {
+			SetTheme (pLocalTheme);
+			delete pTheme;
+		} else {
+			ThemeManager.AddFileTheme (pTheme, GetTitle ());
+			SetTheme (pTheme);
+		}
+	}
+
 	m_bIsLoading = true;
 	node = root->children;
 	while (node) {
@@ -698,7 +728,9 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 		else
 			throw (int) 0;
 	}
-	
+
+	if (!m_Theme->Save (xml))
+		throw (int) 0;
 	if (!SaveChildren (xml, xml->children))
 		throw 1;
 
@@ -879,7 +911,7 @@ void gcpDocument::Remove(const char* Id)
 
 void gcpDocument::OnProperties()
 {
-	if (!m_DocPropDlg) m_DocPropDlg = new gcpDocPropDlg(this);
+	new gcpDocPropDlg(this);
 }
 
 void gcpDocument::SetTitle(const gchar* title)
@@ -939,6 +971,7 @@ void gcpDocument::OnUndo()
 	m_bUndoRedo = false;
 	Update ();
 	EmptyTranslationTable ();
+	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
 }
 
 void gcpDocument::OnRedo()
@@ -958,22 +991,25 @@ void gcpDocument::OnRedo()
 	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_bUndoRedo = false;
 	EmptyTranslationTable ();
+	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
 }
 
 void gcpDocument::FinishOperation()
 {
 	if (!m_pCurOp) return;//FIXME: should at least emit a warning
 	m_UndoList.push_front(m_pCurOp);
-	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
 	while (!m_RedoList.empty())
 	{
 		delete m_RedoList.front();
 		m_RedoList.pop_front();
 	}
-	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
 	m_pCurOp = NULL;
-	m_bDirty = true;
-	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
+	SetDirty (true);
+	if (m_Window) {
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
+	}
 	Update ();
 }
 
@@ -989,9 +1025,10 @@ void gcpDocument::PopOperation()
 	{
 		delete m_UndoList.front();
 		m_UndoList.pop_front();
-		if (m_UndoList.empty())
+		if (m_UndoList.empty() && m_Window)
 			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", false);
 	}
+	SetDirty (m_LastStackSize != m_UndoList.size () || (m_LastStackSize > 0 && m_OpID != m_UndoList.front ()->GetID ()));
 }
 
 void gcpDocument::PushOperation(gcpOperation* operation, bool undo)
@@ -1127,9 +1164,14 @@ void gcpDocument::ExportImage (string const &filename, const char* type, int res
 void gcpDocument::SetReadOnly (bool ro)
 {
 	m_bReadOnly = ro;
-	if (m_pApp->GetActiveDocument () == this) {
-		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/Save", !ro);
-		m_Window->ActivateActionWidget ("/MainToolbar/Save", !ro);
+	if (!ro && (m_FileType != "application/x-gchempaint")) {
+		OBFormat *f = OBConversion::FormatFromMIME (m_FileType.c_str ());
+		m_bReadOnly = (f == NULL)? true: f->Flags () & NOTWRITABLE;
+	}
+	m_bUndoRedo = true;
+	if (m_Window) {
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/Save", !m_bReadOnly);
+		m_Window->ActivateActionWidget ("/MainToolbar/Save", !m_bReadOnly);
 	}
 }
 
@@ -1149,7 +1191,10 @@ double gcpDocument::GetYAlign ()
 
 void gcpDocument::SetTheme (gcpTheme *theme)
 {
+	if (m_Theme)
+		m_Theme->RemoveClient (this);
 	m_Theme = theme;
+	m_Theme->AddClient (this);
 	m_BondLength = theme->GetBondLength ();
 	m_BondAngle = theme->GetBondAngle ();
 	m_ArrowLength = theme->GetArrowLength ();
@@ -1167,3 +1212,85 @@ void gcpDocument::SetTheme (gcpTheme *theme)
 	pango_attr_list_insert (m_PangoAttrList, pango_attr_stretch_new (theme->GetFontStretch ()));
 	pango_attr_list_insert (m_PangoAttrList, pango_attr_variant_new (theme->GetFontVariant ()));
 }
+
+bool gcpDocument::OnSignal (SignalId Signal, Object *Child)
+{
+	if (Signal == OnThemeChangedSignal) {
+		m_BondLength = m_Theme->GetBondLength ();
+		m_BondAngle = m_Theme->GetBondAngle ();
+		m_ArrowLength = m_Theme->GetArrowLength ();
+		m_TextFontFamily = m_Theme->GetTextFontFamily ();
+		m_TextFontStyle = m_Theme->GetTextFontStyle ();
+		m_TextFontWeight = m_Theme->GetTextFontWeight ();
+		m_TextFontVariant = m_Theme->GetTextFontVariant ();
+		m_TextFontStretch = m_Theme->GetTextFontStretch ();
+		m_TextFontSize = m_Theme->GetTextFontSize ();
+		pango_attr_list_unref (m_PangoAttrList);
+		m_PangoAttrList = pango_attr_list_new ();
+		pango_attr_list_insert (m_PangoAttrList, pango_attr_family_new (m_Theme->GetFontFamily ()));
+		pango_attr_list_insert (m_PangoAttrList, pango_attr_style_new (m_Theme->GetFontStyle ()));
+		pango_attr_list_insert (m_PangoAttrList, pango_attr_weight_new (m_Theme->GetFontWeight ()));
+		pango_attr_list_insert (m_PangoAttrList, pango_attr_stretch_new (m_Theme->GetFontStretch ()));
+		pango_attr_list_insert (m_PangoAttrList, pango_attr_variant_new (m_Theme->GetFontVariant ()));
+		m_pView->UpdateTheme ();
+	}
+	return false;
+}
+
+void gcpDocument::SetDirty (bool isDirty)
+{
+	if (!m_Window)
+		return;
+	char *title = g_strdup_printf ((isDirty? "*%s": "%s"), GetTitle ());
+	m_Window->SetTitle (title);
+	g_free (title);
+	Document::SetDirty (isDirty);
+}
+
+void gcpDocument::SetLabel(const gchar* label)
+{
+	m_label = g_strdup (label);
+	m_Window->SetTitle (label);
+}
+
+void gcpDocument::OnThemeNamesChanged ()
+{
+	gcpDocPropDlg *dlg = dynamic_cast <gcpDocPropDlg *> (GetDialog ("properties"));
+	if (dlg)
+		dlg->OnThemeNamesChanged ();
+}
+
+double gcpDocument::GetMedianBondLength ()
+{
+	vector <double> lengths;
+	int max = 128, nb = 0;
+	lengths.reserve (max);
+	double result = 0.;
+	stack<map< string, Object * >::iterator> iters;
+	map< string, Object * >::iterator m;
+	Object *Cur = this, *Ob = GetFirstChild (m);
+	while (Ob) {
+		if (Ob->GetType () == BondType) {
+			if (nb == max) {
+				max += 128;
+				lengths.resize (max);
+			}
+			lengths[nb++] = static_cast <gcpBond *> (Ob)->Get2DLength ();
+		} else if (Ob->HasChildren ()) {
+			Cur = Ob;
+			iters.push (m);
+			Ob = Cur->GetFirstChild (m);
+			continue;
+		}
+		Ob = Cur->GetNextChild (m);
+		while (!Ob && !iters.empty ()) {
+			m = iters.top ();
+			iters.pop ();
+			Cur = Cur->GetParent ();
+			Ob = Cur->GetNextChild (m);
+		}
+	}
+	if (nb > 0)
+		go_range_median_inter_nonconst (lengths.data (), nb, &result);
+	return result;
+}
