@@ -4,7 +4,7 @@
  * GChemPaint library
  * mesomery.cc 
  *
- * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,7 +30,7 @@
 #include "mesomery.h"
 #include "mesomery-arrow.h"
 #include "mesomer.h"
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #include <set>
 
@@ -107,16 +107,6 @@ bool gcpMesomery::Load (xmlNodePtr node)
 	return true;
 }
 
-void gcpMesomery::Add (GtkWidget* w)
-{
-	map<string, Object*>::iterator i;
-	Object* p = GetFirstChild (i);
-	while (p) {
-		p->Add (w);
-		p = GetNextChild (i);
-	}
-}
-
 typedef struct
 {
 	double x, y;
@@ -556,7 +546,7 @@ void gcpMesomery::Align ()
 		// align the molecule
 		if ((*j).second->GetStartMesomer () != pStart)
 			(*j).second->Reverse ();
-		DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding () * pTheme->GetZoomFactor (), pTheme->GetZoomFactor ());
+		DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding (), pTheme->GetZoomFactor ());
 		positions[(*j).first] = md;
 	}
 	/* now add one mesomer to each growing chain and terminate chains when the end is
@@ -630,7 +620,7 @@ void gcpMesomery::Align ()
 				} else {
 					md.chain = md0.chain;
 					md.chain->mesomers.push_back ((*j).first);
-					DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding () * pTheme->GetZoomFactor (), pTheme->GetZoomFactor ());
+					DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding (), pTheme->GetZoomFactor ());
 					positions[(*j).first] = md;
 				}
 				break;
@@ -691,7 +681,7 @@ void gcpMesomery::Align ()
 					} else {
 						md.chain = md0.chain;
 						md.chain->mesomers.push_back ((*j).first);
-						DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding () * pTheme->GetZoomFactor (), pTheme->GetZoomFactor ());
+						DoAlign ((*j).second, md0, md, pTheme->GetArrowPadding (), pTheme->GetZoomFactor ());
 						positions[(*j).first] = md;
 					}
 				}
@@ -715,7 +705,7 @@ void gcpMesomery::Align ()
 			arrow = static_cast<gcpMesomeryArrow*> (pObj);
 			md = positions[arrow->GetStartMesomer ()];
 			md0 = positions[arrow->GetEndMesomer ()];
-			AlignArrow (arrow, md, md0, pTheme->GetArrowPadding () * pTheme->GetZoomFactor (), pTheme->GetZoomFactor ());
+			AlignArrow (arrow, md, md0, pTheme->GetArrowPadding (), pTheme->GetZoomFactor ());
 		}
 	pView->Update (this);
 }
