@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,17 +30,24 @@
 #include "application.h"
 #include "tool.h"
 #include "text.h"
+#include "theme.h"
+#include "window.h"
 #include "libgcpcanvas/gprintable.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
+#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <math.h>
 #include <unistd.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomeprint/gnome-print-job.h>
 #include <locale.h>
-#include <fstream>
-
-extern bool new_ui;
+#ifdef HAVE_FSTREAM
+#	include <fstream>
+#else
+#	include <fstream.h>
+#endif
 
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
@@ -167,53 +174,30 @@ using namespace std;
 gcpView::gcpView (gcpDocument *pDoc, bool Embedded)
 {
 	m_pDoc = pDoc;
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	m_PangoFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoFontDesc, DefaultFontSize);
+	pango_font_description_set_family (m_PangoFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoFontDesc, pTheme->GetFontSize ());
 	m_sFontName = pango_font_description_to_string (m_PangoFontDesc);
 	m_PangoSmallFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoSmallFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoSmallFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoSmallFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoSmallFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoSmallFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoSmallFontDesc, DefaultFontSize * 2 / 3);
+	pango_font_description_set_family (m_PangoSmallFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoSmallFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoSmallFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoSmallFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoSmallFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoSmallFontDesc, pTheme->GetFontSize () * 2 / 3);
 	m_sSmallFontName = pango_font_description_to_string (m_PangoSmallFontDesc);
-	m_PangoTinyFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoTinyFontDesc, DefaultFontFamily);
-	pango_font_description_set_style (m_PangoTinyFontDesc, DefaultFontStyle);
-	pango_font_description_set_weight (m_PangoTinyFontDesc, DefaultFontWeight);
-	pango_font_description_set_variant (m_PangoTinyFontDesc, DefaultFontVariant);
-	pango_font_description_set_stretch (m_PangoTinyFontDesc, DefaultFontStretch);
-	pango_font_description_set_size (m_PangoTinyFontDesc, DefaultFontSize / 2);
-	m_sTinyFontName = pango_font_description_to_string (m_PangoTinyFontDesc);
-	m_DefaultPadding = DefaultPadding;
-	m_DefaultStereoBondWidth = DefaultStereoBondWidth;
-	m_DefaultHashWidth = DefaultHashWidth;
-	m_DefaultHashDist = DefaultHashDist;
-	m_DefaultBondWidth = DefaultBondWidth;
-	m_DefaultBondLength = DefaultBondLength;
-	m_DefaultBondDist = DefaultBondDist;
-	m_DefaultArrowLength = DefaultArrowLength;
-	m_DefaultArrowHeadA = DefaultArrowHeadA;
-	m_DefaultArrowHeadB = DefaultArrowHeadB;
-	m_DefaultArrowHeadC = DefaultArrowHeadC;
-	m_DefaultArrowDist = DefaultArrowDist;
-	m_DefaultObjectPadding = DefaultObjectPadding;
-	m_DefaultSignPadding = DefaultSignPadding;
-	m_ArrowPadding = DefaultArrowPadding;
-	m_StoichiometryPadding = DefaultStoichiometryPadding;
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
-	if (!(m_bEmbedded = Embedded) && !new_ui)
-		CreateNewWidget ();
+	m_bEmbedded = Embedded;
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
+	m_pWidget = NULL;
 }
 
 gcpView::~gcpView ()
@@ -225,13 +209,13 @@ gcpView::~gcpView ()
 		g_free (m_sSmallFontName);
 	pango_font_description_free (m_PangoFontDesc);
 	pango_font_description_free (m_PangoSmallFontDesc);
-	pango_font_description_free (m_PangoTinyFontDesc);
 	g_object_unref (m_UIManager);
 }
 
 bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 {
 	gcpApplication *App = m_pDoc->GetApplication ();
+	gcpTheme *pTheme = m_pDoc->GetTheme ();
 	gcpTool* pActiveTool = App? App->GetActiveTool (): NULL;
 	if ((!m_pDoc->GetEditable ()) || (!pActiveTool))
 		return true;
@@ -259,7 +243,7 @@ bool gcpView::OnEvent(GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 				if ((*i).first->GetType() == BondType)
 				{
 					pBond = (gcpBond*)(*i).first;
-					if (pBond->GetDist(x / m_pData->ZoomFactor, y / m_pData->ZoomFactor) < (m_DefaultPadding + m_DefaultBondWidth / 2) / m_pData->ZoomFactor)
+					if (pBond->GetDist(x / pTheme->GetZoomFactor (), y / pTheme->GetZoomFactor ()) < (pTheme->GetPadding () + pTheme->GetBondWidth () / 2) / pTheme->GetZoomFactor ())
 					{
 						item = GNOME_CANVAS_ITEM((*i).second);
 						break;
@@ -355,21 +339,6 @@ GtkWidget* gcpView::CreateNewWidget()
 		m_pData->View = this;
 		gnome_canvas_set_pixels_per_unit(GNOME_CANVAS(m_pWidget), 1);
 		gnome_canvas_set_scroll_region(GNOME_CANVAS(m_pWidget), 0, 0, m_width, m_height);
-		m_pData->ZoomFactor = DefaultZoomFactor;
-		m_pData->Padding = m_DefaultPadding * DefaultZoomFactor;
-		m_pData->BondWidth = m_DefaultBondWidth * DefaultZoomFactor;
-		m_pData->BondLength = m_DefaultBondLength * DefaultZoomFactor;
-		m_pData->BondDist = m_DefaultBondDist * DefaultZoomFactor;
-		m_pData->ArrowLength = m_DefaultArrowLength * DefaultZoomFactor;
-		m_pData->ArrowHeadA = m_DefaultArrowHeadA * DefaultZoomFactor;
-		m_pData->ArrowHeadB = m_DefaultArrowHeadB * DefaultZoomFactor;
-		m_pData->ArrowHeadC = m_DefaultArrowHeadC * DefaultZoomFactor;
-		m_pData->ArrowDist = m_DefaultArrowDist * DefaultZoomFactor;
-		m_pData->HashWidth = m_DefaultHashWidth * DefaultZoomFactor;
-		m_pData->HashDist = m_DefaultHashDist * DefaultZoomFactor;
-		m_pData->StereoBondWidth = m_DefaultStereoBondWidth * DefaultZoomFactor;
-		m_pData->ObjectPadding = m_DefaultObjectPadding * DefaultZoomFactor;
-		m_pData->SignPadding = m_DefaultSignPadding * DefaultZoomFactor;
 		m_pData->Zoom = 1.0;
 		m_pData->Background = gnome_canvas_item_new(
 									gnome_canvas_root(GNOME_CANVAS(m_pWidget)),
@@ -394,29 +363,18 @@ GtkWidget* gcpView::CreateNewWidget()
 		if (pWidget)
 		{
 			gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(pWidget), "data");
-			std::map<Object*, GnomeCanvasGroup*>::iterator i;
-			for (i = pData->Items.begin(); i != pData->Items.end(); i++)
+			std::map<Object*, GnomeCanvasGroup*>::iterator i, iend = pData->Items.end();
+			for (i = pData->Items.begin(); i != iend; i++)
+				if ((*i).first->GetType () != BondType)
+				 (*i).first->Add(m_pWidget);
+			for (i = pData->Items.begin(); i != iend; i++)
+				if ((*i).first->GetType () == BondType)
 				 (*i).first->Add(m_pWidget);
 		}
 		else
 		{
 			m_PangoContext = gtk_widget_create_pango_context(m_pWidget);
 			g_object_ref(G_OBJECT(m_PangoContext));
-			// bad hack to get charge sign size.
-			if (ChargeSignSize == 0.) {
-				PangoLayout *pl = pango_layout_new (m_PangoContext);
-				char *markup = g_strdup_printf ("<span font_desc=\"%s\">\xE2\x8a\x95</span>", m_sSmallFontName);
-				pango_layout_set_markup (pl, markup, -1);
-				g_free (markup);
-				PangoLayoutIter* iter = pango_layout_get_iter (pl);
-				PangoRectangle ink, log;
-				pango_layout_iter_get_run_extents (iter, &ink, &log);
-				ChargeYAlign = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
-				pango_layout_iter_free (iter);
-				g_object_unref (pl);
-				ChargeSignSize = ink.width / PANGO_SCALE;
-				ChargeSignHeight = log.height / PANGO_SCALE;
-			}
 			UpdateFont();
 		}
 	}
@@ -481,7 +439,7 @@ GnomeCanvasItem* gcpView::GetBackground()
 
 double gcpView::GetZoomFactor()
 {
-	return m_pData->ZoomFactor;
+	return m_pDoc->GetTheme ()->GetZoomFactor ();
 }
 
 void gcpView::UpdateFont()
@@ -496,7 +454,7 @@ void gcpView::UpdateFont()
 	pl = pango_layout_new(m_PangoContext);
 	pango_layout_set_text(pl, "C", 1);
 	pango_layout_get_extents(pl, &rect, NULL);
-	m_BaseLineOffset =  (double(rect.height / PANGO_SCALE) / 2.0) / DefaultZoomFactor;
+	m_BaseLineOffset =  (double(rect.height / PANGO_SCALE) / 2.0) / m_pDoc->GetTheme ()->GetZoomFactor ();
 	g_object_unref(G_OBJECT(pl));
 }
 	
@@ -514,16 +472,6 @@ void gcpView::Remove(Object* pObject)
 	}
 }
 
-void gcpView::UpdateLabel(const gchar* label)
-{
-	GtkLabel* pLabel = (GtkLabel*)g_object_get_data(G_OBJECT(m_pWidget), "label");
-	if (pLabel) gtk_label_set_text(pLabel, label);
-	GtkContainer* pMenu = (GtkContainer*)g_object_get_data(G_OBJECT(m_pWidget), "menu");
-	if (!pMenu) return;
-	GList* l = gtk_container_get_children(pMenu);
-	if (GTK_IS_LABEL(l->data)) gtk_label_set_text((GtkLabel*)(l->data), label);
-}
-
 void gcpView::OnDeleteSelection (GtkWidget* w)
 {
 	m_pWidget = w;
@@ -573,15 +521,20 @@ void gcpView::OnDeleteSelection (GtkWidget* w)
 		}
 	}
 	m_pDoc->FinishOperation ();
-	pApp->ActivateMenu ("Copy", false);
-	pApp->ActivateMenu ("Cut", false);
-	pApp->ActivateMenu ("Erase", false);
+	gcpWindow *Win = m_pDoc->GetWindow ();
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 }
 
 GtkTargetEntry const targets[] = {
 	{(char *) GCHEMPAINT_ATOM_NAME,  0, 0},
-	{(char *)"UTF8_STRING", 0, 1},
-	{(char *)"STRING", 0, 2}
+	{(char *) "image/svg",  0, 1},
+	{(char *) "image/svg+xml",  0, 2},
+	{(char *) "image/png",  0, 3},
+	{(char *) "image/jpeg",  0, 4},
+	{(char *) "UTF8_STRING", 0, 1},
+	{(char *) "STRING", 0, 2}
 };
 
 void gcpView::OnCopySelection(GtkWidget* w, GtkClipboard* clipboard)
@@ -611,47 +564,34 @@ void gcpView::OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_dat
 	if (!pActiveTool || (pActiveTool != pApp->GetTool("Select"))) return;
 	xmlDocPtr xml;
 	m_pData->UnselectAll();
-	switch (*DataType)
-	{
+	switch (*DataType) {
 		case 0:
-			xml = xmlParseMemory((const char*)selection_data->data, selection_data->length);
-			m_pDoc->AddData(xml->children->children);
-			xmlFreeDoc(xml);
+			xml = xmlParseMemory ((const char*) selection_data->data, selection_data->length);
+			m_pDoc->AddData (xml->children->children);
+			xmlFreeDoc (xml);
 			break;
-		case 1:
-			{
-				gcpText* text = new gcpText();
-				GtkTextBuffer *buf = (GtkTextBuffer*)text->GetTextBuffer();
-				gtk_text_buffer_set_text(buf, (const char*)selection_data->data, selection_data->length);
-				GtkTextIter start, end;
-				gtk_text_buffer_get_bounds(buf, &start, &end);
-				gtk_text_buffer_remove_all_tags(buf, &start, &end);
-				gtk_text_buffer_apply_tag_by_name(buf, pApp->GetFontName(), &start, &end);
-				text->OnChanged(buf);
-				m_pDoc->AddObject(text);
-				m_pData->SetSelected(text);
+		case 5: {
+				gcpText* text = new gcpText ();
+				PangoLayout *layout = text->GetLayout ();
+				pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+				text->OnChanged (true);
+				m_pDoc->AddObject (text);
+				m_pData->SetSelected (text);
 			}
 			break;
-		case 2:
-			{
-				gcpText* text = new gcpText();
-				GtkTextBuffer *buf = (GtkTextBuffer*)text->GetTextBuffer();
-				if (!g_utf8_validate( (const char*)selection_data->data, selection_data->length, NULL))
-				{
+		case 6: {
+				gcpText* text = new gcpText ();
+				PangoLayout *layout = text->GetLayout ();
+				if (!g_utf8_validate ((const char*) selection_data->data, selection_data->length, NULL)) {
 					gsize r, w;
-					gchar* newstr = g_locale_to_utf8((const char*)selection_data->data, selection_data->length, &r, &w, NULL);
-					gtk_text_buffer_set_text(buf, newstr, w);
-					g_free(newstr);
-				}
-				else
-					gtk_text_buffer_set_text(buf, (const char*)selection_data->data, selection_data->length);
-				GtkTextIter start, end;
-				gtk_text_buffer_get_bounds(buf, &start, &end);
-				gtk_text_buffer_remove_all_tags(buf, &start, &end);
-				gtk_text_buffer_apply_tag_by_name(buf, pApp->GetFontName(), &start, &end);
-				text->OnChanged(buf);
-				m_pDoc->AddObject(text);
-				m_pData->SetSelected(text);
+					gchar* newstr = g_locale_to_utf8 ((const char*) selection_data->data, selection_data->length, &r, &w, NULL);
+					pango_layout_set_text (layout, newstr, w);
+					g_free (newstr);
+				} else
+					pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+				text->OnChanged (true);
+				m_pDoc->AddObject (text);
+				m_pData->SetSelected (text);
 			}
 			break;
 	}
@@ -716,9 +656,10 @@ void gcpView::OnCutSelection(GtkWidget* w, GtkClipboard* clipboard)
 		OnCopySelection(w, clipboard);
 		OnDeleteSelection(w);
 	}
-	pApp->ActivateMenu("Copy", false);
-	pApp->ActivateMenu("Cut", false);
-	pApp->ActivateMenu("Erase", false);
+	gcpWindow *Win = m_pDoc->GetWindow ();
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 }
 
 bool gcpView::OnKeyPress(GtkWidget* w, GdkEventKey* event)
@@ -798,13 +739,13 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 	{
 		x1 = - x1;
 		y1 = - y1;
-		gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(m_pWidget), "data");
-		m_pDoc->Move(x1 / pData->ZoomFactor, y1 / pData->ZoomFactor);
+		gcpTheme* pTheme = m_pDoc->GetTheme ();
+		m_pDoc->Move(x1 / pTheme->GetZoomFactor (), y1 / pTheme->GetZoomFactor ());
 		Update(m_pDoc);
 	}
 }
 
-void gcpView::SetGnomeCanvasRichTextActive(GnomeCanvasRichTextExt* item)
+void gcpView::SetGnomeCanvasPangoActive (GnomeCanvasPango* item)
 {
 	m_ActiveRichText = item;
 	m_Dragging = false;
@@ -827,7 +768,23 @@ void gcpView::OnSelectAll()
 		pActiveTool->AddSelection(m_pData);
 }
 
-void gcpView::ExportImage (const char* filename, const char* type)
+static gboolean do_save_image (const gchar *buf, gsize count, GError **error, gpointer data)
+{
+	GnomeVFSHandle *handle = (GnomeVFSHandle*) data;
+	GnomeVFSFileSize written = 0;
+	GnomeVFSResult res;
+	while (count) {
+		res = gnome_vfs_write (handle, buf, count, &written);
+		if (res != GNOME_VFS_OK) {
+			g_set_error (error, g_quark_from_static_string ("gchempaint"), res, gnome_vfs_result_to_string (res));
+			return false;
+		}
+		count -= written;
+	}
+	return true;
+}
+
+void gcpView::ExportImage (string const &filename, const char* type, int resolution)
 {
 	ArtDRect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
@@ -868,10 +825,10 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		gnome_print_config_unref(config);
 		char buf[256];
 		ifstream *fin = new ifstream (tmpname);
-		ofstream *fout = new ofstream (filename);
-		*fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
-		*fout << "%%BoundingBox: " << (int) (rect.x0 * .75 + ml) << " " << (int) (hp - mt - rect.y1 * .75)  <<  " "  << (int) (rect.x1 * .75 + ml) << " " << (int) (hp - mt - rect.y0 * .75) << endl;
-		*fout << "%%HiResBoundingBox: " << rect.x0 * .75 + ml << " " << hp - mt - rect.y1 * .75 <<  " "  << rect.x1 * .75 + ml << " " << hp - mt - rect.y0 * .75 << endl;
+		ostringstream fout (filename.c_str ());
+		fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
+		fout << "%%BoundingBox: " << (int) (rect.x0 * .75 + ml) << " " << (int) (hp - mt - rect.y1 * .75)  <<  " "  << (int) (rect.x1 * .75 + ml) << " " << (int) (hp - mt - rect.y0 * .75) << endl;
+		fout << "%%HiResBoundingBox: " << rect.x0 * .75 + ml << " " << hp - mt - rect.y1 * .75 <<  " "  << rect.x1 * .75 + ml << " " << hp - mt - rect.y0 * .75 << endl;
 		fin->getline (buf, 256);
 		while (!fin->eof ()) {
 			fin->getline (buf, 256);
@@ -907,8 +864,8 @@ void gcpView::ExportImage (const char* filename, const char* type)
 				else if (!strncmp (buf + 2, "PageResources", strlen ("PageResources")))
 					continue;
 				else if (!strncmp (buf + 2, "EndComments", strlen ("EndComments"))) {
-					 *fout << buf << endl;
-					if (!strcmp (filename + strlen (filename) - 5, ".epsi")) {
+					 fout << buf << endl;
+					if (!filename.compare (filename.length () - 5, 5, ".epsi")) {
 						gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
 						GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
 						GnomeCanvasBuf cbuf;
@@ -924,161 +881,189 @@ void gcpView::ExportImage (const char* filename, const char* type)
 						(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &cbuf);
 						/* use 8 bits depth, no more than 250 bytes per line*/
 						lines = (w + 249) / 250 * h;
-						*fout << "%%BeginPreview: " << w << " " << h << " " << 8 << " " << lines << endl;
-						*fout << hex;
+						fout << "%%BeginPreview: " << w << " " << h << " " << 8 << " " << lines << endl;
+						fout << hex;
 						for (j = 0; j < h; j++) {
 							bytes = 0;
 							for (i = 0; i < 3 * w; i+= 3) {
 								if (bytes == 0)
-									*fout << "%";
+									fout << "%";
 								mask = 0xff - (cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3;
-								*fout << (mask & 0xf);
+								fout << (mask & 0xf);
 								mask = mask >> 4;
-								*fout << mask;
+								fout << mask;
 								bytes++;
 								if (bytes == 250) {
-									*fout << endl;
+									fout << endl;
 									bytes = 0;
 								}
 							}
 							if (bytes)
-								*fout << endl;
+								fout << endl;
 							cbuf.buf += cbuf.buf_rowstride;
 						}
 						/* use 1 bit depth, no more than 128 bytes per line (max width 1024) */
 /*						lines = (w + 1023) / 1024 * h;
-						*fout << "%%BeginPreview: " << w << " " << h << " " << 1 << " " << lines << endl;
-						*fout << hex;
+						fout << "%%BeginPreview: " << w << " " << h << " " << 1 << " " << lines << endl;
+						fout << hex;
 						for (j = 0; j < h; j++) {
 							mask = 1; b = 0;
 							bytes = 0;
 							for (i = 0; i < 3 * w; i+= 3) {
 								if (bytes == 0)
-									*fout << "%";
+									fout << "%";
 								if ((cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3 < 0x80)
 									b |= mask;
 								mask <<= 1;
 								if (mask == 16) {
-									*fout << (int) b;
+									fout << (int) b;
 									mask = 1;
 									b = 0;
 									bytes++;
 									if (bytes == 128) {
-										*fout << endl;
+										fout << endl;
 										bytes = 0;
 									}
 								}
 							}
 							if (mask != 1) {
-								*fout << (int) b;
+								fout << (int) b;
 								bytes++;
 							}
 							if (bytes)
-								*fout << endl;
+								fout << endl;
 							cbuf.buf += cbuf.buf_rowstride;
 						}*/
-						*fout << dec;
-						*fout << "%%EndPreview" << endl;
+						fout << dec;
+						fout << "%%EndPreview" << endl;
 						g_object_unref (pixbuf);
 					}
 				} else if (!strncmp (buf + 2, "BeginProlog", strlen ("BeginProlog"))) {
-					 *fout << buf << endl;
-					 *fout << "save" << endl;
-					 *fout << "countdictstack" << endl;
-					 *fout << "mark" << endl;
-					 *fout << "newpath" << endl;
-					 *fout << "/showpage {} def" << endl;
-					 *fout << "/setpagedevice {pop} def" << endl;
-					 *fout << "%%EndProlog" << endl;
-					 *fout << "%%Page 1 1" << endl;
+					 fout << buf << endl;
+					 fout << "save" << endl;
+					 fout << "countdictstack" << endl;
+					 fout << "mark" << endl;
+					 fout << "newpath" << endl;
+					 fout << "/showpage {} def" << endl;
+					 fout << "/setpagedevice {pop} def" << endl;
+					 fout << "%%EndProlog" << endl;
+					 fout << "%%Page 1 1" << endl;
 				} else if (!strncmp (buf + 2, "Trailer", strlen ("Trailer"))) {
-					 *fout << buf << endl;
-					 *fout << "cleartomark" << endl;
-					 *fout << "countdictstack" << endl;
-					 *fout << "exch sub { end } repeat" << endl;
-					 *fout << "restore" << endl;
+					 fout << buf << endl;
+					 fout << "cleartomark" << endl;
+					 fout << "countdictstack" << endl;
+					 fout << "exch sub { end } repeat" << endl;
+					 fout << "restore" << endl;
 				} else 
-					 *fout << buf << endl;
+					 fout << buf << endl;
 			} else
-			 *fout << buf << endl;
+			 fout << buf << endl;
 		}
 		fin->close ();
 		delete fin;
-		fout->close ();
-		delete fout;
+		GnomeVFSHandle *handle = NULL;
+		GnomeVFSFileSize n;
+		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK)
+			gnome_vfs_write (handle, fout.str ().c_str (), (GnomeVFSFileSize) fout.str ().size (), &n);
 	} else if (!strcmp (type, "svg")) {
-		xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		char *buf;
-		setlocale (LC_NUMERIC, "C");
-		xmlNewDtd (doc,
-			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
-			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
-		xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
-		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
-		xmlSetNs (doc->children, ns);
-		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
-		rect.x0 = floor (rect.x0);
-		rect.y0 = floor (rect.y0);
-		rect.x1 = ceil (rect.x1);
-		rect.y1 = ceil (rect.y1);
-		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-		xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
-		g_free (buf);
-		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-		xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
-		g_free (buf);
-		xmlNodePtr node;
-		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
-		xmlAddChild (doc->children, node);
-		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-		xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
-		g_free (buf);
-		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-		xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
-		g_free (buf);
-		xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
-		xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
-		if (rect.x0 != 0. || rect.y0 != 0) {
-			node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
-			xmlAddChild (doc->children, node);
-			buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
-			xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
-			g_free (buf);
-		} else
-			node = doc->children;
-		g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
+		xmlDocPtr doc = BuildSVG ();
 		xmlIndentTreeOutput = true;
 		xmlKeepBlanksDefault (0);
-		xmlSaveFormatFile (filename, doc, true);
+		xmlSaveFormatFile (filename.c_str (), doc, true);
 		xmlFreeDoc (doc);
 	} else {
+		double zoom;
+		if (resolution > 0) {
+			int screenres = m_pDoc->GetApp ()->GetScreenResolution ();
+			zoom = (double) resolution / screenres;
+			w = (int) rint ((double) w * zoom);
+			h = (int) rint ((double) h * zoom);
+		} else
+			zoom = 1.;
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
 		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
 		GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
 		gdk_pixbuf_fill (pixbuf, 0xffffffff);
 		GnomeCanvasBuf buf;
 		buf.buf = gdk_pixbuf_get_pixels (pixbuf);
-		buf.rect.x0 = (int) floor (rect.x0);
-		buf.rect.x1 = (int) ceil (rect.x1);
-		buf.rect.y0 = (int) floor (rect.y0);
-		buf.rect.y1 = (int) ceil (rect.y1);
+		buf.rect.x0 = (int) floor (rect.x0 * zoom);
+		buf.rect.x1 = (int) ceil (rect.x1 * zoom);
+		buf.rect.y0 = (int) floor (rect.y0 * zoom);
+		buf.rect.y1 = (int) ceil (rect.y1 * zoom);
 		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 		buf.bg_color = 0xffffff;
 		buf.is_buf = 1;
 		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
-		gdk_pixbuf_save (pixbuf, filename, type, NULL, NULL);
+		GnomeVFSHandle *handle = NULL;
+		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK) {
+			GError *error = NULL;
+			gdk_pixbuf_save_to_callbackv (pixbuf, do_save_image, handle, type, NULL, NULL, &error);
+			if (error) {
+				cerr << _("Unable to save image file: ") << error->message << endl;
+				g_error_free (error);
+			}
+			gnome_vfs_close (handle); // hope there will be no error there
+		}
 		g_object_unref (pixbuf);
+		// restore zoom level
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), m_pData->Zoom);
 	}
 	m_pData->ShowSelection (true);
 }
 
+xmlDocPtr gcpView::BuildSVG ()
+{
+	ArtDRect rect;
+	m_pData->GetObjectBounds (m_pDoc, &rect);
+	xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
+	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	char *buf;
+	setlocale (LC_NUMERIC, "C");
+	xmlNewDtd (doc,
+		   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
+		   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
+	xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
+	xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
+	xmlSetNs (doc->children, ns);
+	xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
+	rect.x0 = floor (rect.x0);
+	rect.y0 = floor (rect.y0);
+	rect.x1 = ceil (rect.x1);
+	rect.y1 = ceil (rect.y1);
+	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+	xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
+	g_free (buf);
+	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+	xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
+	g_free (buf);
+	xmlNodePtr node;
+	node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
+	xmlAddChild (doc->children, node);
+	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+	xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
+	g_free (buf);
+	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+	xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
+	g_free (buf);
+	xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
+	xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
+	if (rect.x0 != 0. || rect.y0 != 0) {
+		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
+		xmlAddChild (doc->children, node);
+		buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
+		xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
+		g_free (buf);
+	} else
+		node = doc->children;
+	g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+	return doc;
+}
+
 void gcpView::EnsureSize ()
 {
 	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
-	while (canvas->idle_id)
-		gtk_main_iteration();
 	gnome_canvas_update_now (canvas);
 	if (GTK_WIDGET_REALIZED (m_pWidget))
 		g_signal_emit_by_name (m_pWidget, "update_bounds");
@@ -1089,4 +1074,40 @@ void gcpView::Zoom (double zoom)
 	m_pData->Zoom = zoom;
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
 	EnsureSize ();
+	// Call OnSize to be certain that the canvas scroll region will be correct
+	OnSize (m_pWidget, m_width, m_height);
+}
+
+void gcpView::ShowCursor (bool show)
+{
+	if (m_ActiveRichText)
+		g_object_set (G_OBJECT (m_ActiveRichText), "editing", show, NULL);
+}
+
+void gcpView::UpdateTheme ()
+{
+	if (m_sFontName)
+		g_free (m_sFontName);
+	if (m_sSmallFontName)
+		g_free (m_sSmallFontName);
+	pango_font_description_free (m_PangoFontDesc);
+	pango_font_description_free (m_PangoSmallFontDesc);
+	gcpTheme *pTheme = m_pDoc->GetTheme ();
+	m_PangoFontDesc = pango_font_description_new ();
+	pango_font_description_set_family (m_PangoFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoFontDesc, pTheme->GetFontSize ());
+	m_sFontName = pango_font_description_to_string (m_PangoFontDesc);
+	m_PangoSmallFontDesc = pango_font_description_new ();
+	pango_font_description_set_family (m_PangoSmallFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoSmallFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoSmallFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoSmallFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoSmallFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoSmallFontDesc, pTheme->GetFontSize () * 2 / 3);
+	m_sSmallFontName = pango_font_description_to_string (m_PangoSmallFontDesc);
+	Update (m_pDoc);
 }
