@@ -307,7 +307,7 @@ static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
 	  { "New", GTK_STOCK_NEW, N_("_New File"), NULL,
 		  N_("Create a new file"), G_CALLBACK (on_file_new) },
-	  { "NewThemed", GTK_STOCK_NEW, N_("_New File with Theme..."), "<shift><control>N",
+	  { "NewThemed", GTK_STOCK_NEW, N_("Ne_w File with Theme..."), "<shift><control>N",
 		  N_("Create a new file using a theme"), G_CALLBACK (on_themed_file_new) },
 	  { "Open", GTK_STOCK_OPEN, N_("_Open..."), "<control>O",
 		  N_("Open a file"), G_CALLBACK (on_file_open) },
@@ -534,7 +534,7 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *Theme, char const *extra_
 	gtk_recent_filter_add_mime_type (filter, "chemical/x-pdb");
 	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (w), filter);
 	g_signal_connect (G_OBJECT (w), "item-activated", G_CALLBACK (on_recent), this);
-	GtkWidget *item = gtk_menu_item_new_with_label (_("Open recent"));
+	GtkWidget *item = gtk_menu_item_new_with_mnemonic (_("Open _recent"));
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), w);
 	gtk_widget_show_all (item);
 	gtk_menu_shell_insert (GTK_MENU_SHELL (gtk_widget_get_parent (menu)), item, 3);
