@@ -22,11 +22,13 @@
  * USA
  */
 
+#include "gchempaint-config.h"
 #include "retrosynthesisstep.h"
 #include "retrosynthesis.h"
 #include "retrosynthesisarrow.h"
 #include "lib/molecule.h"
 #include "lib/document.h"
+#include <glib/gi18n-lib.h>
 
 TypeId RetrosynthesisStepType;
 
