@@ -4,7 +4,7 @@
  * GChemPaint atoms plugin
  * elementtool.cc 
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -43,45 +43,47 @@ gcpElementTool::~gcpElementTool()
 
 bool gcpElementTool::OnClicked()
 {
-	int CurZ = m_pApp->GetCurZ() ;
-	if (m_pObject)
-	{
-		if (m_pObject->GetType() != AtomType) return false; 
-		if 	(((gcpAtom*)m_pObject)->GetTotalBondsNumber() > (int) Element::GetMaxBonds(CurZ)) return false;
-		((gcpAtom*)m_pObject)->GetCoords(&m_x0, &m_y0);
+	int CurZ = m_pApp->GetCurZ ();
+	if (m_pObject) {
+		if (m_pObject->GetType () != AtomType)
+			return false; 
+		if 	(((gcpAtom*) m_pObject)->GetTotalBondsNumber () > (int) Element::GetMaxBonds (CurZ))
+			return false;
+		((gcpAtom*) m_pObject)->GetCoords (&m_x0, &m_y0);
 		m_x0 *= m_dZoomFactor;
 		m_y0 *= m_dZoomFactor;
 	}
 	m_bChanged = true;
-	const gchar* symbol = gcu::Element::Symbol(CurZ);
-	PangoLayout* pl = pango_layout_new(m_pView->GetPangoContext());
-	pango_layout_set_text(pl, symbol, strlen(symbol));
-	gint width = pango_layout_get_width(pl);
+	const gchar* symbol = gcu::Element::Symbol (CurZ);
+	PangoLayout *pl = pango_layout_new (m_pView->GetPangoContext ());
+	pango_layout_set_font_description (pl, m_pView->GetPangoFontDesc ());
+	pango_layout_set_text (pl, symbol, strlen (symbol));
+	gint width = pango_layout_get_width (pl);
 	double padding = m_pView->GetDoc ()->GetTheme ()->GetPadding ();
-	m_x1 = m_x0 - (double)width / 2 - padding;
+	m_x1 = m_x0 - (double) width / 2 - padding;
 	m_y1 = m_y0 - m_pView->GetFontHeight() / 2 - padding;
-	m_x2 = m_x0 + (double)width / 2 + padding;
-	m_y2 = m_y0 + m_pView->GetFontHeight() / 2 + padding;
-	m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
-	gnome_canvas_item_new(
-					(GnomeCanvasGroup*)m_pItem,
-					gnome_canvas_rect_get_type(),
+	m_x2 = m_x0 + (double) width / 2 + padding;
+	m_y2 = m_y0 + m_pView->GetFontHeight () / 2 + padding;
+	m_pItem = gnome_canvas_item_new (m_pGroup, gnome_canvas_group_ext_get_type (), NULL);
+	gnome_canvas_item_new (
+					(GnomeCanvasGroup*) m_pItem,
+					gnome_canvas_rect_get_type (),
 					"x1", m_x1,
 					"y1", m_y1,
 					"x2", m_x2,
 					"y2", m_y2,
 					"fill_color", "white",
 					NULL);
-	gnome_canvas_item_new(
-					(GnomeCanvasGroup*)m_pItem,
-					gnome_canvas_text_get_type(),
-					"text", symbol,
-					"x", rint(m_x0),
-					"y", rint(m_y0),
-					"font", m_pView->GetFontName(),
+	gnome_canvas_item_new (
+					(GnomeCanvasGroup*) m_pItem,
+					gnome_canvas_pango_get_type (),
+					"layout", pl,
+					"x", rint (m_x0),
+					"y", rint (m_y0),
 					"anchor", GTK_ANCHOR_CENTER,
 					"fill_color", AddColor,
 					NULL);
+	g_object_unref (pl);
 	return true;
 }
 
@@ -150,8 +152,3 @@ void gcpElementTool::OnRelease ()
 		pDoc->FinishOperation ();
 	}
 }
-
-void gcpElementTool::SetOptions()
-{
-//	m_pApp->OnShowElements ();
-}
