@@ -4,7 +4,7 @@
  * GChemPaint bonds plugin
  * bondtool.cc
  *
- * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -499,7 +499,7 @@ void gcpBondTool::SetLength (double length)
 
 GtkWidget *gcpBondTool::GetPropertyPage ()
 {
-	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/bond.glade", "bond", GETTEXT_PACKAGE);
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint/ui/bond.glade", "bond", GETTEXT_PACKAGE);
 	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
 	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
 	m_AngleBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-angle"));
